// Mirko Igel
// ueberarbeitet: frankp

alias( Folge            = analysis::Folge ):
alias( Funktion         = analysis::Funktion ):
alias( zusFunktion      = analysis::zusFunktion ):
alias( FunkTerm(f)      = extop(f, 1) ):
alias( FunkVar(f)       = extop(f, 2) ):

Funktion := newDomain( "analysis::Funktion" ):
Funktion::info := "Ein Datentyp fuer reelle Funktionen":

Funktion::new := proc(x, f)
begin
    if args(0) <> 2 then
        error("Erwarte 2 Argumente: x und einen Term in x")
    elif domtype(x) <> DOM_IDENT then
        error("Erwarte einen Bezeichner als 1. Argument")
    elif not testtype(f, Type::Arithmetical) then 
        error("Erwarte einen Term vom Typ 'Type::Arithmetical' als 2. Argument")
    else
        assume(x,Type::Real,_and);
        return( new(Funktion, f, x) )
    end_if
end_proc:

Funktion::expr2text:= f -> expr2text(FunkVar(f))." :-> ".expr2text(FunkTerm(f)):

Funktion::print:= f -> FunkTerm(f):

if version() = [2,0,0] then 
  Funktion::Content:= FAIL
else
  Funktion::Content := proc(Out, f)
  begin
    Out::Capply(Out::Clambda, Out(FunkVar(f)), Out(FunkTerm(f))) 
  end_proc:
end_if:

Funktion::neueLV := proc(f, x)
begin
    return(Funktion(x, subs(FunkTerm(f), FunkVar(f) = x)));
end_proc:

Funktion::expr := f -> FunkTerm(f):

Funktion::ersetzen := proc(f)
begin
    new(Funktion, eval(subs(FunkTerm(f), (args(2..args(0))))), FunkVar(f));
end_proc:

Funktion::subs := proc(f)
begin
    new(Funktion, (subs(FunkTerm(f), (args(2..args(0))))), FunkVar(f));
end_proc:

Funktion::has := f -> has(FunkTerm(f),args(2..args(0))):

Funktion::hastype := f -> hastype(FunkTerm(f),args(2..args(0))):

Funktion::einsetzen := proc(f,s)
    local temp;
begin
    if domtype(s) = Funktion then
        temp := eval( subs(FunkTerm(f), FunkVar(f) = FunkTerm(s)) );
    else
        temp := eval( subs(FunkTerm(f),FunkVar(f) = s) );
    end_if;
    return(temp)
end_proc:

Funktion::erweitern := proc(f, erw)
    local z, n;
begin
    if sign((erw := analysis::deEqualize(erw))) = 0 then
        error("Erweiterung muss ungleich Null sein");
    end;
    z := expand(numer(FunkTerm(f)) * erw);
    n  := expand(denom(FunkTerm(f)) * erw);
    return(new(Funktion, z / n, FunkVar(f)))
end_proc:

Funktion::_fconcat := proc(f,s)
    local i, tmp;
begin
    case args(0) 
    of 1 do 
       return(f);
    of 2 do
       if domtype(f) = Funktion then
           case domtype(s)
           of Folge    do
               f := Funktion::neueLV(f, op(s,2));
               return(Folge::new(FunkVar(f), eval(subs(FunkTerm(f), FunkVar(f) = op(s,1)))  ));
           of Funktion do
               f := Funktion::neueLV(f, FunkVar(s));
               s := FunkTerm(s);
               return(Funktion::new(FunkVar(f), eval(subs(FunkTerm(f), FunkVar(f) = s))  ));
           otherwise
               return(Funktion::new(FunkVar(f), eval(subs(FunkTerm(f), FunkVar(f) = s))  ));
           end_case:
       elif domtype(s) = Funktion then
           return(Funktion::new(FunkVar(s), f(FunkTerm(s))));
       else
           error("unerlaubter Operand")
       end_if;
   otherwise
       tmp := _fconcat(f,s);
       for i from 3 to args(0) do
           tmp := hold(_fconcat) (tmp @ args(i));
       end_for;
       return( eval(tmp) );
   end_case
end_proc:

Funktion::func_call := proc(f,s)
begin
    f:= context(f); s:= context(s);
    if domtype(s) = Funktion then
        return(f@s);
    elif domtype(s) = Folge then
        return(f@s);
    else
        if not testtype(s, Type::Arithmetical) then
            error("Erwarte Term vom Typ 'Type::Arithmetical'")
        else
            return( eval(subs(FunkTerm(f),FunkVar(f) = s)) )
        end_if
    end_if
end_proc:

Funktion::_negate := f -> new(Funktion, -FunkTerm(f), FunkVar(f)):

Funktion::_plus := proc(f1,f2)
    local tmp, i;
begin
    case args(0) 
    of 1 do 
        return( f1 )
    of 2 do
        if domtype(f1) = Folge or domtype(f1) = zusFunktion then
            error("Ungueltiger Typ");
        elif domtype(f2) = Folge or domtype(f2) = zusFunktion then
            error("Ungueltiger Typ");
        elif domtype(f1) <> Funktion then
            f1 := Funktion(FunkVar(f2), f1);
        elif domtype(f2) <> Funktion then
            f2 := Funktion(FunkVar(f1), f2);
        end_if;
        return(Funktion(FunkVar(f1), _plus(FunkTerm(f1), FunkTerm(f2))));
    otherwise
        tmp := _plus(f1,f2);
        for i from 3 to args(0) do
            tmp := hold(_plus) (tmp + args(i));
        end_for;
        return( eval(tmp) );
    end_case;
end_proc:

Funktion::_subtract := proc(f1,f2)
begin
    if domtype(f2) = Folge or domtype(f2) = zusFunktion then
        error("Ungueltiger Typ");
    elif domtype(f2) <> Funktion then
        f2 := Funktion(FunkVar(f1), f2);
    end_if;
    return(Funktion(FunkVar(f1), _subtract(FunkTerm(f1), FunkTerm(f2))));
end_proc:

Funktion::_mult := proc(f1,f2)
    local i, tmp;
begin
    case args(0) 
    of 1 do 
        return(f1);
    of 2 do
        if domtype(f1) = Folge or domtype(f1) = zusFunktion then
            error("Ungueltiger Typ");
        elif domtype(f2) = Folge or domtype(f2) = zusFunktion then
            error("Ungueltiger Typ");
        elif domtype(f1) <> Funktion then
            f1 := Funktion(FunkVar(f2), f1);
        elif domtype(f2) <> Funktion then
            f2 := Funktion(FunkVar(f1), f2);
        end_if;
        return(Funktion(FunkVar(f1), _mult(FunkTerm(f1), FunkTerm(f2))));
    otherwise
        tmp := _subtract(f1,f2);
        for i from 3 to args(0) do
           tmp := hold(_mult)(tmp * args(i));
        end:
        return( eval(tmp) );
    end_case;
end_proc:

Funktion::_divide := proc(f1,f2)    
begin                  
    if domtype(f2) = Funktion then
        if sign(FunkTerm(f2)) = 0 then
            error("Division durch Null");
        end:
    elif sign(f2) = 0 then;
        error("Division durch Null")
    end_if;

    if domtype(f1) <> Funktion and domtype(f2) = Funktion  then
        return(Funktion(FunkVar(f2), f1 / FunkVar(f2)));
    elif domtype(f1) = Funktion and domtype(f2) <> Funktion then
        return(Funktion(FunkVar(f1), FunkTerm(f1) / f2));
    else
        f2 := Funktion::neueLV(f2, FunkVar(f1)):
        return(new(Funktion, FunkTerm(f1) / FunkTerm(f2), FunkVar(f1)))
    end_if:
end_proc:

Funktion::_power := proc(f, n)
begin
    if contains( {Funktion,Folge,zusFunktion},domtype(n) ) or
       not testtype(n,Type::Arithmetical) 
    then
        error("Ungueltiger Exponent")
    else
        return(new(Funktion, FunkTerm(f)^n, FunkVar(f)))
    end_if
end_proc:

Funktion::indets := f -> indets(FunkTerm(f)):

Funktion::limit := f -> limit(FunkTerm(f), args(2..args(0))):

Funktion::discont := proc(f)
begin
    return(discont(FunkTerm(f), FunkVar(f), args(3..args(0))));
end_proc:

Funktion::D := proc(f)
begin
    if args(0) <> 1 then 
        error("Erwarte nur ein Parameter")
    end_if;

    return(new( Funktion, diff(FunkTerm(f), FunkVar(f)),FunkVar(f) ))
end_proc:

Funktion::diff := proc(f, x)
begin
    if args(0) = 1 then x := FunkVar(f) end_if;
    return( Funktion::new( FunkVar(f),diff(FunkTerm(f), x, args(3..args(0))) ))
end_proc:

Funktion::int := proc(f)
begin
    case args(0)
    of 1 do
        return(Funktion::new(FunkVar(f), int(FunkTerm(f), FunkVar(f))));
    of 2 do
        if type(args(2)) = "_equal" then
            return(int(FunkTerm(f), args(2..args(0))));
        else
            return(Funktion::new(FunkVar(f), int(FunkTerm(f), args(2..args(0)))));
        end_if
    otherwise
        error("Falsche Anzahl an Argumenten")
    end_case
end_proc:

Funktion::graph := proc(f, r)
begin
    if args(0) < 2 then
        error("Erwarte mindestens 2 Parameter")
    end_if;

    r := analysis::deEqualize(r);
    if type(r) <> "_range"  then
        error("Erwarte einen Bereich der Form 'a..b' als 2. Parameter")
    else
        return(plot::Function2d( FunkTerm(f),FunkVar(f)=r,args(3..args(0)) ))
    end_if
end_proc:

Funktion::map := proc(f)
begin
    return(op(map([f], args(2..args(0)))))
end_proc:

Funktion::Name             := "analysis::Funktion": // damit domtype(f)=Funktion liefert.
Funktion::domtype          := FAIL:       // wird ueber 'Name' geregelt.
Funktion::type             := FAIL:
Funktion::evaluate         := FAIL:
Funktion::posteval         := FAIL:
Funktion::subsex           := FAIL:
Funktion::slot             := FAIL:
Funktion::undefinedEntries := FAIL:
Funktion::allEntries       := FAIL:
Funktion::whichEntry       := FAIL:
Funktion::new_extelement   := FAIL:
Funktion::_index           := FAIL:
Funktion::solve            := FAIL:
Funktion::nops             := FAIL:
Funktion::op               := FAIL:
Funktion::float            := FAIL:
Funktion::factor           := FAIL:
Funktion::poly             := FAIL:

/************************************************************************
make_slot -- create default slot

make_slot(e,f) creates a slot f for the domain e (here: = Funktion) by 
converting the slot to a functional expression; arguments of type 'Funktion' 
are converted into expressions before the function f is called. The result 
of function calls of the so created slot is an object of type Function.

NOTE: This MUST be the last method defined for Function.
************************************************************************/
Funktion::args2expr:= proc(x) 
begin 
    if args(0) = 1 then
        if domtype(x) = Funktion then
            return(extop(x,1))
        else
            return(x)
        end_if
    else
        return(
            op( map([args()], x -> if domtype(x) = Funktion then extop(x,1) else x end) )
        )
    end_if
end_proc:

Funktion::make_slot:= proc(e,s) 
begin 
    if domtype(s) = DOM_STRING then
        return(eval(text2expr(s)) @ Funktion::args2expr)
    else
        return(FAIL)
    end_if
end_proc:

