alias ( zusFunktion = analysis::zusFunktion ):
alias ( Funktion    = analysis::Funktion ):
alias ( Folge       = analysis::Folge ):
alias ( FunkTerm(f)  = op(f,1) ):
alias ( FunkVar(f)  = op(f,2) ):

zusFunktion := newDomain("analysis::zusFunktion"):
zusFunktion::info := "Ein Datentyp fuer zusammengesetzte reelle Funktionen":

zusFunktion::new := proc(x)
    local intervs, i, liste, tmp;
begin
    if args(0) < 2 then
        error("Erwarte mindestens zwei Argumente")
    end_if;
    if args(0) = 2 then
        return( Funktion::new(args()) )
    end_if;

    liste := [args(2..args(0))];

    intervs := [];
    for i from 2 to nops(liste) step 2 do
        if type(liste[i]) = "_range" then
            liste[i] := Dom::Interval( [op(liste[i])] );
        elif type(float(liste[i])) = DOM_FLOAT then
            liste[i] := Dom::Interval([liste[i]], [liste[i]]);
        elif domtype(liste[i]) <> Dom::Interval then
            error("Erwarte ein Intervall vom Typ 'Dom::Interval' im ".expr2text(i).". Parameter");
        end_if:
        intervs := append(intervs, (liste[i]));
    end_for;
    if not analysis::pairwiseIntersect(intervs) then
        error("Die Definitionsbereiche der Teilfunktionen ueberdecken sich")
    end_if;

    for i from 1 to nops(liste) step 2 do
        if domtype(liste[i]) <> Funktion then
            if not testtype(liste[i],Type::Arithmetical) then
                error("Erwarte Terme vom Typ 'Type::Arithmetical'")
            elif (tmp:= op(analysis::zuFunktionen(liste[i]))) <> FAIL then
                liste[i] := tmp;
            else
                liste[i] := Funktion::new(x, liste[i]);  
            end:
        end_if:
        //        if FunkVar(liste[i]) <> x then
        //            error("Falsche Laufvariable in der ".expr2text((i div 2) + 1).". Teilfunktion");
        //        end:
    end:
    return(new(zusFunktion, op(liste)));
end_proc:

zusFunktion::print := proc(f)
    local i, o;
    begin
        o := array(1..(nops(f) div 2), 1..2);
        for i from 1 to (nops(f)-1) step 2 do
            o[(i + 1) div 2, 1] := extop(f, i);
            o[(i + 1) div 2, 2] := hold(_in)(FunkVar(extop(f, i)), extop(f, i+1));
        end_for;
       return(o);
    end_proc:

// Typesetting output
if version() = [2,0,0] then 
  zusFunktion::Content:= FAIL
else
  zusFunktion::Content :=
  proc(Out, f)
    local i, x;
  begin
      if version() = [2,0,0] then 
        return(FAIL):
      else 
        x:= FunkVar(extop(f,1));
        Out::Cpiecewise(
                     (Out(extop(f,2*i-1)),
                     Out::Cpwcondition(Out::Cin(Out(x),Out(extop(f, 2*i))))
                     )
           $ i = 1..extnops(f) div 2
        )
      end_if:
  end_proc:
end_if:

zusFunktion::limit := proc()
begin
    error("Wende die Grenzwertbestimmung auf die entsprechende Teilfunktion an");
end_proc:

zusFunktion::graph :=
proc(f, range)
  local i, fi, options, rangeIV, rangei, section, _section,
  points_included, points_excluded;
begin
  if args(0) < 2 then
    error("Erwarte mindestens zwei Argumente");
  elif type(range) <> "_equal" then 
    error("2. Parameter muss ein Bereich sein");
  else
    options:= args(3..args(0));
  end_if;

  range:= rhs(range);
  rangeIV := Dom::Interval( [op(range, 1)],[op(range, 2)] );
  
  fi:= zusFunktion::funktionen(f);
  rangei:= zusFunktion::intervalle(f);

  points_included := [];
  points_excluded := [];
  for i from 1 to nops(fi) do
    f := fi[i];
    section := rangeIV intersect rangei[i];
    _section:= expr(section);
    if type(_section) = "_range" then
      fi[i] := plot::Function2d( FunkTerm(f),FunkVar(f)=_section,options );
      
      if contains(section, op(_section, 1)) then
        points_included := append(points_included,
                                  [op(_section, 1), f(op(_section, 1))]);
      else
        points_excluded := append(points_excluded,
                                  [op(_section, 1), f(op(_section, 1))]);
      end_if;
      if contains(section, op(_section, 2)) then
        points_included := append(points_included,
                                  [op(_section, 2), f(op(_section, 2))]);
      else
        points_excluded := append(points_excluded,
                                  [op(_section, 2), f(op(_section, 2))]);
      end_if
    else
      fi[i]:= FAIL; // nicht zeichnen!
      if nops(_section) > 0 then
        points_included := append(points_included,
                                  [op(_section, 1), f(op(_section, 1))]);
      end_if
    end_if
  end_for;
  fi:= subs(fi,FAIL = null()); // lsche alle nicht zu zeichnenden Objekte


  if points_included <> [] then
    fi := append(fi,
                 plot::PointList2d(points_included,
                                   hold(Color) = (fi[1])::Color,
                                   hold(PointStyle) = hold(FilledCircles)
                                   )
                 )
  end_if;
  if points_excluded <> [] then
    fi := append(fi,
                 plot::PointList2d(points_excluded,
                                   hold(Color) = (fi[1])::Color,
                                   hold(PointStyle) = hold(Circles)
                                   )
                 )
  end_if;

  return(plot::Group2d( op(fi) ))
end_proc:

zusFunktion::intervalle := proc(f)
    local liste, i;
begin
    liste := [];
    for i from 2 to nops(f) step 2 do
        liste := append(liste, op(f, i));
    end_for;
    return(liste);
end_proc:

zusFunktion::funktionen := proc(f)
    local liste, i;
begin
    liste := [];
    for i from 1 to nops(f) step 2 do
        liste := append(liste, op(f, i));
    end_for:
    return(liste);
end_proc:

zusFunktion::diff := proc(f)
    local i, liste;
begin
    if args(0) <> 1 then 
        error("Erwarte nur ein Argument")
    end_if;

    liste := [];
    for i from 1 to nops(f)-1 step 2 do
        liste := append(liste, diff((op(f,i)),FunkVar((op(f,i)))), op(f,i+1));
   end_for;

    new(zusFunktion, op(liste));
end_proc:   

zusFunktion::expr := proc(f)
    local i, liste;
begin
    liste := [];
    for i from 1 to nops(f) step 2 do
        liste := append(liste, FunkTerm(op(f,i)));
    end_for;
    return(liste);
end_proc:

zusFunktion::func_call := proc(f, s)
    local rueckgabe, wert, i, bereich, funk;
begin
    s := context(s);
    rueckgabe := [];
    for i from 1 to nops(f) step 2 do
        bereich := op(f, i+1);
        funk := op(f, i);
        if domtype(bereich) = Dom::Interval or type(bereich) = "_union" then
            wert := contains(bereich, s);
            if wert then
                rueckgabe := append(rueckgabe, funk(s));
            end_if;
        elif type(bereich) = DOM_SET and float(op(bereich)) = float(s)  then
            rueckgabe := append(rueckgabe, funk(s));
        end_if;
    end_for;
    if rueckgabe = [] then
        error("Die Funktion ist an der Stelle ". expr2text(s)." nicht definiert");
    elif nops(rueckgabe) > 1 then
        error("Die Funktion ist an der Stelle ". expr2text(s)." nicht eindeutig");
    else
        return(op(rueckgabe));
    end_if;
end_proc:

zusFunktion::teilFunktionen := proc(f)
    local funks, intervs, i, liste;
begin
    funks   := zusFunktion::funktionen(f);
    intervs := zusFunktion::intervalle(f);
    liste   := [];

    for i from 1 to nops(funks) do
        liste := append(liste, zusFunktion(FunkVar(funks[i]), FunkTerm(funks[i]), intervs[i]));
    end_for;
    return(liste);
end_proc:

