// Markus Thewes
// ueberarbeitet: frankp, 20.03.2000

alias ( ispoly = analysis::istPolynom ):

/**********************************************************************************
    integriere( f,modus=schluessel <,par1, par2> )
**********************************************************************************/
analysis::integriere := proc(funktion,modus,parameter1,parameter2)
    local zfunktion,ex,x,schluessel,rechteSeite1,rechteSeite2,linkeSeite1,linkeSeite2,wert;
begin
    // Argumentenueberpruefung:
    if args(0) < 2 or args(0) > 4 then
        error("Ungueltige Anzahl an Parametern");
    end_if;

    zfunktion := analysis::zuFunktionen(funktion)[1];
    if domtype(zfunktion)<> DOM_FAIL then
        funktion := zfunktion;
        ex := op(funktion,1);
        x := op(funktion,2);
    else  
        error("Der erste Parameter muss eine Funktion oder ein Term in einer Unbekannten sein")
    end_if;

    if type(modus)="_equal" then
        if op(modus,1) <> hold(Modus) then
            error("2. Parameter muss die Form 'Modus = ...' besitzen");
        end_if;
        schluessel := op(modus,2)
    else  
        error("2. Parameter muss die Form 'Modus = ...' besitzen")
    end_if;
      
    case schluessel
    of "PartInt" do
        if args(0) = 3 then
            if type(parameter1)="_equal" then
                if not has( op(parameter1,1),["u","u'","v","v'"]) then
                    error("Ungueltiger 3. Parameter")
                end_if;
                zfunktion := analysis::zuFunktionen(op(parameter1,2))[1];
                if zfunktion <> FAIL then
                    if testtype( op(parameter1,2),Type::Real ) then
                        rechteSeite1 := analysis::Funktion(x, op(parameter1,2) );
                    else  
                        rechteSeite1 := zfunktion;
                    end_if;
                    if op(rechteSeite1,2)<> op(funktion,2) then
                        error("Die Variable der Funktion und die des dritten Parameters muessen uebereinstimmen")
                    end_if
                else  
                    error("Die rechte Seite des 3. Parameters muss ein Funktion oder ein Term in einer Unbekannten sein")
                end_if;
            else  
                error("Ungueltiger 3. Parameter")
            end_if;
        else 
            error("Ungueltiger 3. Parameter")
        end_if;   
        break;
    of "PartialBruch" do
        if args(0) <> 2 then
            error("Falsche Anzahl an Parametern")
        end_if;
        break;
   
    of "Substitution" do
        if args(0) = 4 then  
            if type(parameter1)<> "_equal" then
                error("Erwarte eine Gleichung als 3. Parameter")
            end_if;
            case type(parameter2) 
            of "_equal" do
                rechteSeite1 := op(parameter1,2);
                rechteSeite2 := op(parameter2,2);
                linkeSeite1 := op(parameter1,1);
                linkeSeite2 := op(parameter2,1);
                if domtype(linkeSeite1) <> DOM_IDENT then
                    error("Die linke Seite des 3. Parameters muss ein Bezeichner sein") 
                end_if;
                if domtype(linkeSeite2)<>DOM_IDENT then
                    error("Die linke Seite des 4. Parameters muss ein Bezeichner sein")
                end_if;
                zfunktion := analysis::zuFunktionen(rechteSeite1)[1];
                if zfunktion <> DOM_FAIL then
                    rechteSeite1 := zfunktion;
                else
                    error("Die rechte Seite des 3. Parameters muss ein Term oder eine Funktion in einer Unbekannten sein")
                end_if;
                zfunktion := analysis::zuFunktionen(rechteSeite2)[1];
                if zfunktion <> FAIL then
                    rechteSeite2 := zfunktion;
                else
                    error("Die rechte Seite des 4. Parameters muss ein Term oder eine Funktion in einer Unbekannten sein")
                end_if;
                if linkeSeite1=op(rechteSeite1,2) then
                    error("Variablenkonflikt im 3. Parameter");
                end_if;
                if linkeSeite2=op(rechteSeite2,2) then
                    error("Variablenkonflikt im 4. Parameter");
                end_if;
                if op(rechteSeite1,2) <> linkeSeite2 then
                    error("Variablenkonflikt im 3. und 4. Parameter")
                end_if;
                if op(rechteSeite2,2) <> linkeSeite1 then
                    error("Variablenkonflikt im 3. und 4. Parameter")
                end_if;
                break;
            of DOM_STRING do
                rechteSeite1 := op(parameter1,2);
                linkeSeite1 := op(parameter1,1);
                if domtype(linkeSeite1) <> DOM_IDENT then
                    error("Die linke Seite des 3. Parameters muss ein Bezeichner sein")
                end_if;
                zfunktion := analysis::zuFunktionen(rechteSeite1)[1];
                if zfunktion <> FAIL then
                    rechteSeite1 := zfunktion;
                else
                    error("Die rechte Seite des 3. Parameters muss ein Term oder eine Funktion in einer Veraenderlichen sein")
                end_if;
                if parameter2 <> "Implizit" then
                    error("Erwarte Option 'Implizit'")
                end_if;
                break;
            otherwise
                error("Erwarte die Umkehrfunktion oder die Option 'Implizit'");
            end_case;
        else
            error("Falsche Anzahl an Parametern")
        end_if;
        break;

    of "Summe" do
        if args(0) <> 2 then
            error("Erwarte nur zwei Parametern")
        end_if;
        break
    otherwise
        error("Ungueltiger Modus")
    end_case;

    // Durchfuehren eines bestimmten Verfahrens:
    case schluessel 
    of "PartInt" do
        wert := analysis::int_partielle(ex,x,op(parameter1,1),op(rechteSeite1,1));
        if wert = FAIL then
            error("Kann partielle Integtation nicht durchfuehren")
        else
            return(wert);
        end_if
    of "PartialBruch" do
        wert := analysis::int_partialbruch(ex,x);
        if wert = FAIL then
            error("Partialbruchzerlegung konnte nicht durchgefuehrt werden");
        else
            return(wert)
        end_if;
   of "Substitution" do :
        if parameter2 <> "Implizit" then
            case analysis::substitution_variablen_check(
                linkeSeite1,op(rechteSeite1,1),linkeSeite2,op(rechteSeite2,1),x
            )
            of 1 do
                error("3. Parameter: Variablenkonflikt zwischen der rechten und linken Seite")
            of 2 do
                error("4. Parameter: Variablenkonflikt zwischen der rechten und linken Seite")
            of 3 do
                error("Variablenkonflikt zwischen der Funktion und dem 4. Parameter")
            of FALSE do
                error("Die angegebene Substitution ist in sich nicht schluessig");
            of TRUE do
                return(analysis::int_substitution(
                    ex,x,linkeSeite1,op(rechteSeite1,1),linkeSeite2,op(rechteSeite2,1))
                );
            end_case;      
        elif linkeSeite1 <> x and op(rechteSeite1,2) = x
             or linkeSeite1 = x and op(rechteSeite1,2)<>x
        then  
            return(analysis::int_substitution_implizit(ex,x,linkeSeite1,rechteSeite1));
        else
            error("Variablenkonflikt zwischen der Funktion und der Substitution");
        end_if;  

    of "Summe" do
        if type(ex)="_plus" then
           return(analysis::int_summe(ex,x));
        else
            error("Die uebergebene Funktion ist keine Summe");
        end_if
    end_case
end_proc:

/*---------------------------------------------------------------------------------
    int_summe
---------------------------------------------------------------------------------*/
analysis::int_summe := proc(ex,x)
    local tree,i,intern,xs,n;
begin
    intern := 0;
    xs := expr2text(x);
    n := nops(ex);
    if n = 2 then
        tree := [
            "Summenregel",
            "int(S1 + S2)d".xs." = int S1 d".xs." + int S2 d".xs
        ]
    else
        tree := [
            "Summenregel",
            "int(S1 + ... + S".n.")d".xs." = int S1 d".xs." + ... + int S".n."d".xs
        ]
    end_if;

    for i from 1 to n do
        tree := append(tree,[op(ex,i),"= S".i]);
        intern := intern + hold(int)(op(ex,i),x);
    end_for;
    tree := append(tree,[intern,"Ergebnis"]);
    return( adt::Tree([hold(int)(ex,x),tree]) )
end_proc:


/*---------------------------------------------------------------------------------
    substitution_variablen_check(Z,ZofX,X,XofZ,x)

    ZofX, XofZ : arithm. Ausdruecke
    X, Z       : DOM_IDENT
    x          : Funktionsvariable (DOM_IDENT)

    Ueberprueft, ob die Substitution in sich schluessig ist, d.h. 
    ob ZofX(XofZ) = Z. Es muss gelten: Z = x oder x = X.

    Rueckgabewert:
    TRUE : Substitution ist in sich schluessig
    FALSE: Substitution ist in sich nicht schluessig
    1    : ZofX enthaelt die Unbekannte Z
    2    : XofZ enthaelt die Unbekannte X
    3    : Variablenkonflikt : Die Funktionsvariable x kommt in der Substitution nicht vor
---------------------------------------------------------------------------------*/
analysis::substitution_variablen_check := proc(Z,ZofX,X,XofZ,x)
begin
    if has(ZofX,Z) then 
        return(1) 
    elif has(XofZ,X) then 
        return(2) 
    elif X = x or x = Z then
        if analysis::istgleich(eval(subsex(ZofX, X=XofZ)),Z,Z,hold(Plain)) = TRUE then
            return(TRUE)
        else
            return(FALSE)
        end_if
    else
        return(3)
    end_if
end_proc:

/*---------------------------------------------------------------------------------
    int_substitution

    Substitutionsvariable,Substitutionsausdruck in Abhaengigkeit von der Variablen
    Variable, Substitutionsausdruck in Abhaenigigkeit von der Substitutionsvariablen
    und Funktionsterm und Funktionsvariable, fuer die gilt : subvar = subvar2var(var), 
    var = var2subvar(subvar).

    Fuehrt sie Substitution durch und liefert den entspr. Baum.
---------------------------------------------------------------------------------*/
analysis::int_substitution := proc(ex,x,subvar,subvar2var,var,var2subvar)
    local wert,tree,differential,dvar,dsubvar,rechteSeitediff,subfkt,_var,_subvar,
          int_substitution_vorarbeit;
begin
    int_substitution_vorarbeit := proc(ex,x,subvar,subvar2var,var,var2subvar)
        local subvar_term, differential;
    begin
        if (subvar = x) then
            subvar_term := eval(subsex(ex,x = subvar2var));
            differential := diff(subvar2var,var);     
        end_if;
        if (var = x) then
            subvar_term := eval(subsex(ex,x=var2subvar));
            differential := 1/eval(subsex(diff(subvar2var,x),x=var2subvar));
        end_if;

        return([differential,simplify(subvar_term*differential)]);
    end_proc:

    wert := int_substitution_vorarbeit(ex,x,subvar,subvar2var,var,var2subvar);
    differential := wert[1];
    subfkt := wert[2];
    _var := expr2text(var);
    _subvar := expr2text(subvar);
    dvar := text2expr("d"._var);
    dsubvar := text2expr("d"._subvar);
    if (subvar=x) then
        rechteSeitediff := simplify(dvar*differential);
        tree := [
            "Integration durch Substitution",
            "int( f("._var.") )d"._var." = int( f(g("._subvar.")) * g'("._subvar.") )d"._subvar,
            [subvar2var,"= "._subvar." : Substitution, "._var." = ".expr2text(var2subvar)],
            [text2expr(expr2text(dsubvar)." = ".expr2text(rechteSeitediff)), "Differential"],
            [subfkt,"Integrand der substituierten Funktion"],
            [hold(int)(subfkt,var),"int ueber den Integrand der substituierten Funktion"],
            [hold(subs)(hold(int)(subfkt,var),var=var2subvar),"Ergebnis"]
        ]
    else
        rechteSeitediff := simplify(dsubvar*differential);
        tree := [
            "Integration durch Substitution",
            "int( f(g("._var.")) * g'("._var.") )d"._var." = int( f("._subvar.") )d"._subvar,
            [subvar2var, "= "._subvar." :  Substitution, "._var." = ".expr2text(var2subvar)],
            [text2expr(expr2text(dvar)." = ".expr2text(rechteSeitediff)),"Differential"],
            [subfkt,"Integrand der substituierten Funktion"],
            [hold(int)(subfkt,subvar),"int ueber den Integrand der substituierten Funktion"],
            [hold(subs)(hold(int)(subfkt,subvar),subvar=subvar2var),"Ergebnis"]
        ]
    end_if;

    return( adt::Tree([hold(int)(ex,x),tree]) )
end_proc:

/*---------------------------------------------------------------------------------
    int_substitution_implizit

    Funktion, Variable, implizite Substitutionsgleichung

    Fuehrt die Substitution durch und liefert den Baum fuer die Substitution.
---------------------------------------------------------------------------------*/
analysis::int_substitution_implizit := proc(ex,x,subvar,subvar2var)
    local wert,tree,differential,dvar,dsubvar,rechteSeitediff,subfkt,_x,_subvar, 
          int_substitution_vorarbeit_implizit,var_subvar2var,term_subvar2var,
          _var_subvar2var,dvar2subvar;
begin
    int_substitution_vorarbeit_implizit := proc(ex,x,subvar,var_subvar2var,term_subvar2var)
        local subvar_term,differential,hilfe,hilfe1;
    begin
        if subvar <> x then
            differential := 1/diff(term_subvar2var,x);
            subvar_term := eval(subsex(ex,term_subvar2var=subvar));
        else
            differential := diff(term_subvar2var,var_subvar2var);
            subvar_term := eval(subsex(ex,x=term_subvar2var));                            
        end_if;

        hilfe := subvar_term*differential;
        hilfe1 := hilfe;
        if traperror(factor(hilfe1))=0 then
            hilfe1 := factor(hilfe1);
            if nops(hilfe1) = 3 then
                hilfe := hilfe1[1]*hilfe1[2]^hilfe1[3]
            end_if
        end_if;
        return([differential,simplify(hilfe)])
    end_proc:
            
    var_subvar2var := op(subvar2var,2);
    term_subvar2var := op(subvar2var,2);

    wert := int_substitution_vorarbeit_implizit(ex,x,subvar,var_subvar2var,term_subvar2var);

    differential := wert[1];
    subfkt := wert[2];

    _x := expr2text(x);
    _subvar := expr2text(subvar);
    _var_subvar2var := expr2text(var_subvar2var);

    dvar := text2expr("d"._x);
    dsubvar := text2expr("d"._subvar);
    dvar2subvar := text2expr("d"._var_subvar2var);

    if subvar = x then
        rechteSeitediff := simplify(dvar2subvar*differential);
        tree := [
            "Integration durch Substitution",
            "int( f("._x.") )d"._x." = int( f(g("._subvar.")) * g'("._subvar.") )d"._subvar,
            [subvar2var,"= "._x." : Substitution"],
            [text2expr(expr2text(dvar)." = ".expr2text(rechteSeitediff)),"Differential"],
            [subfkt,"Integrand der substituierten Funktion"],
            [hold(int)(subfkt,var_subvar2var),"int ueber den Integrand der substituierten Funktion"],
            [hold(subs)(hold(int)(subfkt,subvar),subvar=term_subvar2var),"Ergebnis"]
        ]
    else
        rechteSeitediff := simplify(dsubvar*differential);
        tree := [
            "Integration durch Substitution",
            "int(f(g("._x.")) * g'("._x.")) d"._x." = int(f("._subvar.")) d"._subvar,
            [term_subvar2var,"= ".subvar." :  Substitution"],
            [text2expr(expr2text(dvar)." = ".expr2text(rechteSeitediff)),"Differential"],
            [subfkt,"Integrand der substituierten Funktion"],
            [hold(int)(subfkt,subvar),"int ueber den Integrand der substituierten Funktion"],
            [hold(subs)(hold(int)(subfkt,subvar),subvar=term_subvar2var),"Ergebnis"]
        ]
    end_if;

    return( adt::Tree([hold(int)(ex,x),tree]) )
end_proc:

/*---------------------------------------------------------------------------------
    int_partialbruch(f,x)

    f: arithm. Ausdruck
    x: DOM_IDENT

    Durchfuehrung der Partialbruchzerlegung. Rueckgabe ist ein
    Baum fuer die Partialbruchzerlegung oder FAIL, falls die
    Zerlegung nicht gefunden werden kann.
---------------------------------------------------------------------------------*/
analysis::int_partialbruch := proc(ex,x)
    local zaehler,nenner,wert,partialbruch,ganzrational,
          tree,i,fkt,anzahl;
begin
    zaehler := numer(ex);
    nenner := denom(ex);
    if nenner = 1 then
        return(FAIL)
    elif not ispoly(zaehler) or not ispoly(nenner) then
        return(FAIL)
    end_if;

    wert := analysis::partialbruchzerlegung(ex,x);
    ganzrational := wert[1];
    partialbruch := wert[2];
    anzahl := if type(partialbruch)="_plus" then nops(partialbruch) else 1 end_if;
    if analysis::istgleich(ganzrational,0,x,Plain) = TRUE and anzahl = 1 then
        return(adt::Tree( [hold(int)(ex,x),[
            "Integration einer echt gebrochenen rationalen Funktion", 
            "logarithmische Integration, Substitution,..."
            [hold(int)(ex,x),"Ergebnis"]
        ]] ))
    else
        if ganzrational <> 0 then  
            tree := [
                "Integration einer nicht echt-gebrochen-rationalen Funktion",
                "ganzer Anteil + Summe der Partialbrueche",
                [ganzrational,"ganzer Anteil"]
            ]
        elif anzahl = 1 then
            tree := [
                "Integration einer echt gebrochenen Funktion",
                "echtgebrochender Anteil"
            ]
        else
            tree := [
                "Integration einer echt gebrochenen Funktion",
                "Summe ueber die Partialbrueche"
            ];
        end_if;

        if partialbruch <> 0 then
            for i from 1 to anzahl do
                tree := append(tree,
                    [op(partialbruch,i),expr2text(i).". Partialbruch"]
                )
            end_for
        end_if;

        fkt := 0;
        if ganzrational <> 0 then
            fkt := fkt + hold(int)(ganzrational,x);
        end_if;

        if type(partialbruch) = "_plus" then
            for i from 1 to anzahl do
                fkt := fkt + hold(int)(op(partialbruch,i),x); 
            end_for;
        else
            fkt := fkt + hold(int)(partialbruch,x);
        end_if;                    

        return( adt::Tree([hold(int)(ex,x),append(tree,[fkt,"Ergebnis"])]) )
    end_if
end_proc:

/*---------------------------------------------------------------------------------
    int_partielle(f,x)

    f: arithm. Ausdruck
    x: DOM_IDENT


    Durchfuehrung der partiellen Integration liefert Baum fuer die partiellen 
    Integration oder FAIL, wenn die partielle Integration nicht zum Ziel fuehrt.
---------------------------------------------------------------------------------*/
analysis::int_partielle := proc(ex,x,gewaehlt,rechteSeite)
    local u,v,uStrich,vStrich,tree;
begin
    case gewaehlt
    of "u" do
        u := rechteSeite;
        vStrich := simplify(ex/u);
        v := analysis::MuPADint(vStrich,x);
        if v <> FAIL then
            uStrich := diff(u,x);
            break;
        else
            return(FAIL)
        end_if;
    of "v'" do
        vStrich := rechteSeite;
        v := analysis::MuPADint(vStrich,x);
        if v = FAIL then return(FAIL) end_if;
        u := simplify(ex/vStrich);
        uStrich := diff(u,x);
        break;
    of "u'" do
        uStrich := rechteSeite;
        u := analysis::MuPADint(uStrich,x);
        if u <> FAIL then
            v := simplify(ex/uStrich);
            vStrich := diff(v,x);
            break;
        else
            return(FAIL)
        end_if
   of "v" do
        v := rechteSeite;
        uStrich := simplify(ex/v);
        u := analysis::MuPADint(uStrich,x);
        if u <> FAIL then
            vStrich := diff(v,x);
        else
            return(FAIL)
        end_if
    end_case;

    // create tree:
    if gewaehlt = "u" or gewaehlt = "v'" then
        tree := [
            "Partielle Integration",
            "int( u v' )d".expr2text(x)." = u * v - int( u' v )d".expr2text(x)
        ]
    else
        tree := [
            "Partielle Integration",
            " int( u' v )d".expr2text(x)." = u * v - int( u v' )d".expr2text(x)
        ]
    end_if;

    case gewaehlt 
    of "u" do
        tree := append(tree,
            [u,"gewaehltes ".gewaehlt.", ".gewaehlt."' = ".expr2text(uStrich)],
            [vStrich,"= v' , v = ".expr2text(v)]
        );
        break
    of "v" do
        tree := append(tree,
            [v,"gewaehltes ".gewaehlt.", ".gewaehlt."' = ".expr2text(vStrich)],
            [uStrich,"= u' , u = ".expr2text(u)]
        );
        break
    of "u'" do
        tree := append(tree,
            [uStrich,"gewaehltes u', u = ".expr2text(u)],
            [v,"= v , v' = ".expr2text(vStrich)]
        );
        break
    of "v'" do
        tree := append(tree,
            [vStrich,"gewaehltes v', v = ".expr2text(v)],
            [u,"= u , u' = ".expr2text(uStrich)]
        )
    end_case;
       
    tree := append(tree,[u*v,"= u v"]);
    if gewaehlt = "u" or gewaehlt = "v'" then
        tree := append(tree,[uStrich*v,"Restintegrand u' v"]);
        tree := append(tree,[_plus(u*v-hold(int)(uStrich*v,x)),"Ergebnis"]);
    else
        tree := append(tree,[u*vStrich,"Restintegrand u v'"]);
        tree := append(tree,[_plus(u*v-hold(int)(u*vStrich,x)),"Ergebnis"]);
    end_if;
   
    return( adt::Tree([hold(int)(ex,x),tree]) )
end_proc:

/*---------------------------------------------------------------------------------
    MuPADint(f,x)

    f: arithm. Ausdruck
    x: DOM_IDENT

    Fuert die Integration von f mittels der MuPAD Systemfunktion 'int'
    durch. Liefert FAIL, falls die Integration nicht bestimmt werden kann,
    anderenfalls eine Stammfunktion von f.
---------------------------------------------------------------------------------*/
analysis::MuPADint := proc(f,x)
begin
    f := int(f,x);
    if type(f) = "int" then return( FAIL ) else return( f ) end_if
end_proc:

/*------------------------------------------------------------------------------
    polynom_raus(f,x)

    f: arithm. Ausdruck
    x: DOM_IDENT

    Ermittelt saemtliche Polynome im Ausdruck, die sich zu 1 herauskuerzen.
    Liefert FAIL, falls factor-Befehl nicht durchfuehrbar, sonst
    [zu kuerzende Polynome],restfunktion. Das Ergebnis ist [],ex,
    falls keine Polynome gekuerzt werden koennen.
------------------------------------------------------------------------------*/
analysis::polynom_raus := proc(ex,x)
    local zu1gekuerzt,vollstaendig_gekuerzt,polyzu1gekuerzt,poly_vollstaendig_gekuerzt,gekuerzt,gekuerztErg;
begin
    polyzu1gekuerzt := proc(ex,x)
        local z,n,i_z,i_n,gekuerzt;
    begin
        gekuerzt := [];
        z := numer(ex);
        n := denom(ex);
        if traperror( (z:= factor(z)) ) <> 0 or traperror( (n:= factor(n)) ) <> 0 then
            return(FAIL)
        end_if;
      
        for i_z from 2 to nops(z)-1 step 2 do
            for i_n from 2 to nops(n)-1 step 2 do
                if z[i_z] = n[i_n] and z[i_z+1] = n[i_n+1] then
                    gekuerzt := append(gekuerzt,z[i_z]^z[i_z+1]);
                end_if;
            end_for;
        end_for;

        return(gekuerzt)
    end_proc:              

    poly_vollstaendig_gekuerzt := proc(ex,x)
        local z,n,i_z,i_n,gekuerzt,geaendert;
    begin
        z := numer(ex);
        n := denom(ex);
        if traperror( (z:= factor(z)) ) <> 0 or traperror( (n:= factor(n)) ) <> 0 then
            return(FAIL);
        end_if;
           
        gekuerzt := 1;
        geaendert := FALSE;
        for i_z from 2 to nops(z)-1 step 2 do
            for i_n from 2 to nops(n)-1 step 2 do
                if (z[i_z] = n[i_n]) and (z[i_z+1] <> n[i_n+1]) then
                    gekuerzt := gekuerzt * z[i_z]^z[i_z+1]/n[i_n]^n[i_n+1];
                    geaendert := TRUE;
                end_if;           
            end_for;
        end_for;
        if geaendert then
            return((z[1]/n[1])*gekuerzt)
        else
            return(ex)
        end_if
    end_proc:              

    gekuerzt := proc(ex,x,faktor_liste)
        local intern,n,n_new,z,z_new,i,faktor_basis,faktor_exponent,faktor,j;
    begin
        intern := ex;
        for j from 1 to nops(faktor_liste) do
            faktor := faktor_liste[j];
            z := numer(intern);
            n := denom(intern);
            if traperror( (z:= factor(z)) ) <> 0 or traperror( (n:= factor(n)) ) <> 0 then
                n_new := 1;
                z_new := 1;
            else
                return(FAIL)
            end_if;
            
            if type(faktor)="_power" then 
                faktor_basis := op(faktor,1);
                faktor_exponent := op(faktor,2);
            else
                faktor_basis := faktor;
                faktor_exponent := 1;
            end_if;
            
            for i from 2 to nops(n)-1 step 2 do 
                if n[i] <> faktor_basis then
                    n_new := n_new*n[i]^n[i+1];
                elif faktor_exponent <> n[i+1] then
                    n_new := n_new*n[i]^(n[i+1]-faktor_exponent);
                end_if;
            end_for;
            
            for i from 2 to nops(z)-1 step 2 do 
                if z[i]<>faktor_basis then
                    z_new := z_new*z[i]^z[i+1];
                elif faktor_exponent <> z[i+1] then
                    z_new := z_new*z[i]^(z[i+1]-faktor_exponent);
                end_if;
            end_for;
            
            intern := z_new/n_new;
        end_for;
            
        if j=0 then
            return(ex);
        else
            return((factor(numer(ex))[1]/factor(denom(ex))[1])*intern);
        end_if;
    end_proc:

    zu1gekuerzt := polyzu1gekuerzt(ex,x);
    if zu1gekuerzt <> FAIL then
        if zu1gekuerzt = [] then
            vollstaendig_gekuerzt := poly_vollstaendig_gekuerzt(ex,x);
            if vollstaendig_gekuerzt <> FAIL then
                return([],vollstaendig_gekuerzt);
            else
                return(FAIL)
            end_if;
        else
            vollstaendig_gekuerzt := poly_vollstaendig_gekuerzt(ex,x);
            if vollstaendig_gekuerzt <> FAIL then
                gekuerztErg := gekuerzt(ex,x,zu1gekuerzt);
                if gekuerztErg <> FAIL then
                    return(zu1gekuerzt,poly_vollstaendig_gekuerzt(gekuerztErg,x));
                else
                    return(FAIL)
                end_if
            else
                return(FAIL)
            end_if
        end_if;
    else
        return(FAIL)
    end_if
end_proc:

