// Markus Thewes
// ueberarbeitet: frankp, 20.03.2000

alias ( ispoly =  analysis::istPolynom ):

/*---------------------------------------------------------------------------
    tip_partielleIntegration(f,x)

    f: arithm. Ausdruck
    x: DOM_IDENT

    Uberprueft, ob der MuPAD-Ausdruck mit Hilfe der partiellen Integration 
    integriert werden kann. Liefert Wahlmoeglichkeit von u und v' oder FAIL.
---------------------------------------------------------------------------*/
analysis::tip_partielleIntegration := proc(ex,x)
    local faktor1,faktor2,ini,skalar,n,m,ana_n,ana_m,ana,liste1,liste2,spezialFall_ln,
          spezialFall_ln_poly_exponent,tmp,tmp2;
begin
    // Unterprozeduren
    spezialFall_ln := proc(ex,x)
        local zaehler,nenner;
    begin
        if analysis::test_funk(ex,x,"poly")[1] = TRUE then return( TRUE ) end_if;

        zaehler := numer(ex);
        nenner := denom(ex);
        if testtype(zaehler,Type::Real) and analysis::test_funk(zaehler,x,"poly")[1] = TRUE then
            return(TRUE)
        else
            return(FALSE)
        end_if
    end_proc:

    spezialFall_ln_poly_exponent := proc(ex,x,rueckgabe)
        local tmp,zaehler,nenner;
    begin
        tmp:= analysis::test_funk(ex,x,"poly");
        if tmp[1] = TRUE then
            return(tmp[rueckgabe])
        end_if;
        zaehler := numer(ex);
        nenner := denom(ex);
        tmp:= analysis::test_funk(nenner,x,"poly");
        if testtype(zaehler,Type::Real) and tmp[1] = TRUE then
            return(tmp[rueckgabe])
        else
            return(FALSE)
        end_if
    end_proc:

    // Beginn des Prozedurkoerpers von tip_partielleIntegration
    liste1 := [];
    liste2 := [];
    faktor1 := 1;  
    faktor2 := 1;  
    skalar := 1;            
    ini := FALSE;        

    // Die Prozedur tip_partielle kann nur Tips fuer Produkte geben, die aus 
    // zwei Faktoren abhaengig von der Variablen bestehen!
    if type(ex)="_mult" and nops(ex) >=3 then
        return(FAIL);
    end_if;

    // Die folgenden Anweisungen bewirken, dass die einzelnen Faktoren des 
    // Produktes in eine bestimmte Reihenfolge gebracht werden, damit beim 
    // Fallabgleich keine Schwierigkeiten mit der Reihenfolge der Faktoren 
    // auftreten.

    case type(ex) 
    of "_power" do
        faktor1 := ex;
        faktor2 := 1;
        if not has ( op(faktor1,1),[sin,cos,ln]) then
            return(FAIL)
        end_if;
        // Es wird ueberprueft, ob der Exponent die Proporty PosInt besitzt
        if analysis::check_on_integer( op(analysis::degreep(faktor1,x),2),TRUE) then
            ini := TRUE;
            break
        else 
            return([],[nops(analysis::degreep(faktor1,x)[2])],"property");
        end_if
    of "_mult" do
        if spezialFall_ln(op(ex,1),x) and analysis::test_funk(op(ex,2),x,"ln",1)[1] = TRUE then
            // in Faktor1 steht nun polynom oder (1/polynom)  mit (1/poly) * ln oder poly*ln
            faktor1 := op(ex,1);
            faktor2 := op(ex,2);
            ini := TRUE;
        elif spezialFall_ln(op(ex,2),x) and analysis::test_funk(op(ex,1),x,"ln",1)[1] = TRUE then
            faktor1 := op(ex,2);
            faktor2 := op(ex,1);
            ini := TRUE;
        end_if;
        // Es liegt ein Produkt der Form : sin(a*x+b)^n * cos(c*x+d)^m vor;
        if analysis::test_funk(op(ex,1),x,"sin")[1] = TRUE and analysis::test_funk(op(ex,2),x,"cos")[1] = TRUE then
            faktor1 := op(ex,1);
            faktor2 := op(ex,2);
            liste1 := op(analysis::degreep(faktor1,x),2);       
            liste2 := op(analysis::degreep(faktor2,x),2);       
            // Es wird ueberprueft, ob der Exponent die Proporty PosInt besitzt
            if analysis::check_on_integer( op(analysis::degreep(faktor1,x),2),TRUE) 
               and analysis::check_on_integer( op(analysis::degreep(faktor2,x),2),TRUE) 
            then
                ini := TRUE
            else
                return([],[nops(analysis::degreep(faktor1,x)[2])+nops(analysis::degreep(faktor2,x)[2])],"property");
            end_if
        elif analysis::test_funk(op(ex,2),x,"sin")[1] = TRUE and analysis::test_funk(op(ex,1),x,"cos")[1] = TRUE then
            // Es liegt ein Produkt der Form : cos(a*x+b)^n * sin(c*x+d)^m vor;                       
            faktor1 := op(ex,2);
            faktor2 := op(ex,1);
            liste1 := op(analysis::degreep(faktor1,x),2);       
            liste2 := op(analysis::degreep(faktor2,x),2);       
            // Es wird ueberprueft, ob der Exponent die Proporty PosInt besitzt
            if analysis::check_on_integer( op(analysis::degreep(faktor1,x),2),TRUE) 
               and analysis::check_on_integer( op(analysis::degreep(faktor2,x),2),TRUE)
            then
                ini := TRUE;
            else
                return([],[nops(analysis::degreep(faktor1,x)[2])+nops(analysis::degreep(faktor2,x)[2])],"property");
            end_if;
        elif analysis::test_funk(op(ex,2),x,"sin")[1] = TRUE and analysis::test_funk(op(ex,1),x,"sin")[1] = TRUE
             or analysis::test_funk(op(ex,2),x,"cos")[1] = TRUE and analysis::test_funk(op(ex,1),x,"cos")[1] = TRUE
        then
            // Es liegt ein Produkt der Form : sin(a*x+b)^n * sin(c*x+d)^m vor
            // der Form cos(a*x+b)^n * cos(c*x+d)^m vor  
            faktor1 := op(ex,1);
            faktor2 := op(ex,2);
            liste1 := op(analysis::degreep(faktor1,x),2);       
            liste2 := op(analysis::degreep(faktor2,x),2);       
            // Es wird ueberprueft, ob der Exponent die Proporty PosInt besitzt
            if analysis::check_on_integer( op(analysis::degreep(faktor1,x),2),TRUE)
               and analysis::check_on_integer( op(analysis::degreep(faktor2,x),2),TRUE)
            then
                ini := TRUE;
            else  
                return([],[nops(analysis::degreep(faktor1,x)[2])+nops(analysis::degreep(faktor2,x)[2])],"property");
            end_if;
        end_if;
        // Testen, ob ein Polynom in der Variablen in dem Ausdruck als erster oder zweiter Faktor enthalten ist
        // Das Polynom wird gegebenenfalls in faktor1 geschrieben
        tmp:= analysis::test_funk(op(ex,1),x,"poly");
        if tmp[1] = TRUE and not ini then
            if tmp[3] = "real" and tmp[2] > 0 and domtype(tmp[2]) = DOM_INT or tmp[3] = DOM_IDENT then
                faktor1 := op(ex,1); // Das Polynom
                faktor2 := op(ex,2);
                // Es wird ueberprueft, ob der Exponent die Proporty PosInt besitzt
                if analysis::check_on_integer( op(analysis::degreep(faktor1,x),2),TRUE) then
                    ini := TRUE
                else
                    return([],[nops(analysis::degreep(faktor1,x)[2])],"property")
                end_if
            end_if;  
         else
            tmp:= analysis::test_funk(op(ex,2),x,"poly");
            if tmp[1] = TRUE and not ini then
                if tmp[3] = "real" and tmp[2] > 0 and domtype(tmp[2]) = DOM_INT or tmp[3] = DOM_IDENT then
                    faktor1 := op(ex,2); // Das Polynom
                    faktor2 := op(ex,1);
                    if analysis::check_on_integer( op(analysis::degreep(faktor1,x),2),TRUE ) then
                        ini := TRUE
                    else
                        return([],[nops(analysis::degreep(faktor1,x)[2])],"property")
                    end_if
                end_if
            end_if
        end_if;
        // Testen, ob der Audruck exp(a*x*b) enthaelt
        // Dieser wird dann gegebenfalls in faktor1 geschrieben
        if analysis::test_funk(op(ex,1),x,"exp")[1] = TRUE and not ini then
            faktor1 := op(ex,1);
            faktor2 := op(ex,2);
            ini := TRUE;
        elif analysis::test_funk(op(ex,2),x,"exp")[1] and not ini then
            faktor1 := op(ex,2);
            faktor2 := op(ex,1);
            ini := TRUE;
        end_if;
        break;
    of "ln"     do  
    of "arcsin" do
    of "arccos" do
    of "arctan" do
        faktor1 := ex;
        faktor2 := 1;
        ini := TRUE;
    end_case;

    if not ini then
        // Dieser Fall ist intTip fuer das Schluesselwort "PartInt" unbekannt.
        return(FAIL);
    end_if;

    // Testen auf trig(a*x+b)^n  mit trig = {sin,cos}
    tmp:= analysis::test_funk(faktor1,x,["sin","cos"]);
    if tmp[1] = TRUE and not has(faktor2,x) then
        n := tmp[2]; // Der Exponent
        ana_n := tmp[3];
        
        // Analysieren, ob der Exponent ein Skalar ist, oder nicht                 
        if ana_n = DOM_IDENT then
            return([op(faktor1,1)^(op(faktor1,2)-1),skalar*op(faktor1,1)],[n],[])
        elif ana_n = "real" then
            if n>0 then
                return([op(faktor1,1)^(op(faktor1,2)-1),skalar*op(faktor1,1)],[],[])       
            end_if
        end_if
    end_if;

    // Testen auf trig1(a*x+b)^n * trig2(c*x+d)^m mit trig1,trig2 = {sin,cos}   aber trig1<>trig2
    tmp:= analysis::test_funk(faktor1,x,"sin");
    tmp2:= analysis::test_funk(faktor2,x,"cos");
    if tmp[1] = TRUE and tmp2[1] = TRUE then
        n := tmp[2];       
        m := tmp2[2];
        ana_n := tmp[3];   
        ana_m := tmp2[3];   
        if ana_n = DOM_IDENT and ana_m = DOM_IDENT then
            return([faktor1,faktor2],[n,m],"trig");
        elif ana_n = DOM_IDENT and ana_m = "real" or ana_m = DOM_IDENT and ana_n = "real" then
            if ana_n = DOM_IDENT then
                return([faktor1,faktor2],[n,m],"trig")
            else  
                return([faktor2,faktor1],[m,n],"trig");
            end_if
        elif ana_n = "real" and ana_m = "real" then
            if (n>0) and (m>0) then
                if bool(n>m) then
                    return([faktor1,skalar*faktor2],[],[])
                else
                    return([faktor2,skalar*faktor1],[],[])
                end_if
            end_if
        end_if
    end_if;

    // Testen auf trig1(a*x+b)^n * trig2(c*x+d)^m mit trig1,trig2 = {sin,cos} aber trig1=trig2
    tmp:= analysis::test_funk(faktor2,x,"sin");
    if tmp[1] = TRUE and tmp2[1] = TRUE or 
       analysis::test_funk(faktor1,x,"cos")[1] = TRUE and analysis::test_funk(faktor2,x,"cos")[1] = TRUE
    then
        if tmp[1] = TRUE then ana := "sin" else ana := "cos" end_if;
        
        n := analysis::test_funk(faktor1,x,ana)[2];
        m := analysis::test_funk(faktor2,x,ana)[2];
        ana_n := analysis::test_funk(faktor1,x,ana)[3];
        ana_m := analysis::test_funk(faktor2,x,ana)[3];
        if ana_n = DOM_IDENT and ana_m = DOM_IDENT then
            return([faktor1,faktor2],[n,m],"trig");
        elif ana_n = DOM_IDENT and ana_m = "real" or ana_m = DOM_IDENT and ana_n = "real" then
            if ana_n = DOM_IDENT then
                return([faktor1,faktor2],[n,m],"trig")
            else
                return([faktor2,faktor1],[m,n],"trig")
            end_if;
        elif ana_n = "real" and ana_m = "real" then
            if n > 0 and m > 0 then
                if n > m then
                    return([faktor1,skalar*faktor2],[],[])
                else
                    return([faktor2,skalar*faktor1],[],[])
                end_if
            end_if
        end_if
    end_if;

    // Testen von (polynom)^r * trig(a*x+b)   mit trig = {sin,cos}
    tmp:= analysis::test_funk(faktor1,x,"poly");
    tmp2:= analysis::test_funk(faktor2,x,["sin","cos"],1);
    if tmp[1] = TRUE and tmp2[1] = TRUE then
        if tmp[3] = DOM_IDENT then
            return([faktor1,skalar*faktor2],[tmp[2]],[])
        elif tmp[3] = "real" and tmp[2] > 0 then
            return([faktor1,skalar*faktor2],[],[]);
        end_if
    end_if;

    // Testen von (polynom)^r * exp(a*x+b) 
    tmp2:= analysis::test_funk(faktor2,x,"exp",1);
    if tmp[1] = TRUE and tmp2[1] = TRUE then
        if tmp[3] = DOM_IDENT then
            return([faktor1,faktor2],[tmp[2]],[]);
        elif tmp[2] > 0 then
            return([faktor1,faktor2],[],[]);
        end_if;
    end_if;

    // Testen von (polynom)^r * ln(ax+b)   oder ( 1/ (polynom^r) ) * ln(a*x+b) 
    if spezialFall_ln(faktor1,x) and analysis::test_funk(faktor2,x,"ln",1)[1] = TRUE then
        if spezialFall_ln_poly_exponent(faktor1,x,3)=DOM_IDENT then
            return([faktor2,faktor1],[spezialFall_ln_poly_exponent(faktor1,x,2)],[]);
        elif spezialFall_ln_poly_exponent(faktor1,x,2)>0 then
            return([faktor2,faktor1],[],[]);
        end_if;
    end_if;

    // Testen von exp(a*x+b) * trig(b*x+c)    trig = {sin,cos}
    if analysis::test_funk(faktor1,x,"exp",1)[1] = TRUE 
       and analysis::test_funk(faktor2,x,["sin","cos"],1)[1] = TRUE 
    then
        return([skalar*faktor2,faktor1],[],[]);
    end_if;

    // Testen auf ln(x)^n mit n<>-1 n>0
    tmp:= analysis::test_funk(faktor1,x,"ln");
    if tmp[1] = TRUE and not has(faktor2,x) then
        if tmp[3] = DOM_IDENT then
            return([faktor1,skalar*1],op(analysis::degreep(faktor1,x),2),[]);
        elif tmp[2] > 0 then
            return([faktor1,skalar*1],op(analysis::degreep(faktor1,x),1),[]);
        else
            return(FAIL)
        end_if;
    end_if;

    // Testen auf arcsin(x),arccos(x),arctan(x)
    if analysis::test_funk(faktor1,x,["arcsin","arccos","arctan"],1)[1] = TRUE and not has(faktor2,x) then
        return([faktor1,skalar*faktor2],[],[]);
    end_if;

    return(FAIL)
end_proc:

/*---------------------------------------------------------------------------
    test_funk(f,x,s)

    f: arithm. Ausdruck
    x: DOM_IDENT
    s: DOM_STRING

    Ueberprueft, ob ein bestimmtes mathematisches Konstrukt der Form kontr(linear)^n 
    vorliegt. Das n kann wieder ein Linearkombination von anderen unbestimmten 
    Ausdruecken sein.

    1.Moeglichkeit:  test_funk(ex,x,"poly")
    ex = (polynom1)^j    liefert [TRUE,j],  j = "reak" oder DOM_IDENT
    ex = x^j+x0          liefert [TRUE, j]   j = "real" oder DOM_IDENT
    ex = x^j+x^2+x+1     liefert [TRUE,2]    ueber das j kann man ja i.A. nichts aussagen

    2. Moeglichkeit: test_funk(ex,x,type)
    Prueft ob ex der Form type(linearer Ausdruck)^n vorliegt
    Beispiele:
        ex = sin(2*x+5)^6, test_funk(ex,x,"sin") = [TRUE, 6, "real"]
        ex = sin(2*x+5)^j, test_funk(ex,x,"sin") = [TRUE, j, DOM_IDENT]
        ex = cos(x)      , test_funk(ex,x,"cos") = [FALSE]
---------------------------------------------------------------------------*/
analysis::test_funk := proc(ex,x,anweisungen,teste_auf_expon)
    local basis,exponent,i,n,v;
begin
    if anweisungen = "poly" then
        if type(ex)="_power" then
            basis := op(ex,1);
            exponent := op(ex,2);
            if ispoly(basis,x)and not testtype(basis,Type::Real) then
                if args(0) = 4 then
                    return( [iszero(exponent-teste_auf_expon)] )
                elif testtype(exponent,Type::Real) then
                    if exponent>0 then
                        return( [TRUE,exponent,"real"] )
                    else
                        return( [FALSE] )
                    end_if
                else
                    return( [TRUE,exponent,DOM_IDENT] )
                end_if
            else
                return( [FALSE] )
            end_if;
        elif ispoly(ex,x) then
            if args(0) = 4 then
                return( [iszero(degree(ex)-teste_auf_expon)] )
            else
                return( [TRUE,degree(ex),"real"] )
            end_if
        elif ispoly(poly(ex,[x]),x) then
            n:= analysis::degreep(ex,x);
            v:= op(n,2); n:= op(n,1):
            if n = [0] and nops(v)=1 then
                // Es liegt der Fall x^k+x0 mit Parameter k vor:
                return( [TRUE,op(v,1),DOM_IDENT] )
            elif (n:= degree(ex)) > 0 then
                return( [TRUE,n,"real"] )
            else
                return( [FALSE] )
            end_if
        else
            return( [FALSE] )
        end_if
    else
        if domtype(anweisungen) = DOM_STRING then
            n := 1;
            anweisungen := [anweisungen]
        else
            n := nops(anweisungen)
        end_if;
        for i from 1 to n do
            if type(ex)="_power" then
                basis := op(ex,1);
                exponent := op(ex,2);
                if type(basis) = anweisungen[i] and analysis::linear(op(basis,1),x) then
                    if args(0) = 4 and exponent = teste_auf_expon then
                        return( [TRUE] )
                    elif testtype(exponent,Type::Real) then
                        return( [TRUE,exponent,"real"] ); 
                    else
                        return( [TRUE,exponent,DOM_IDENT] )
                    end_if
                end_if;
            elif type(ex) = anweisungen[i] and analysis::linear(op(ex,1),x) then
                if args(0) = 4 and teste_auf_expon = 1 then
                    return( [TRUE] )
                else
                    return( [TRUE,1,"real"] )
                end_if
            end_if
        end_for;

        return( [FALSE] )
    end_if
end_proc:

/*---------------------------------------------------------------------------
    degreep(f,x)

    f: arithm. Ausdruck
    x: DOM_IDENT

    Bestimmt den Grad eines Polynoms in x und liefert die hoechste 
    skalare Potenz und eine Liste von Parameters in f.

    Beispiel:
        f:= x^3+x^z+5*x^7+x+x^l  mit l,z konstanten
        degreep(f,x) = [7],[l,z]
---------------------------------------------------------------------------*/
analysis::degreep := proc(ex,x)
    local extract_coeffs;
begin
    extract_coeffs:= proc(ex,x)
        local s;
    begin
        case type(ex)
        of DOM_IDENT do
            if ex = x then
                return( [1] )
            else
                return( [ex] )
            end_if
        of "_power" do
            return( [op(ex,2)] )
        of "_mult" do
            // a*x^l
            if testtype(op(ex,[1,2]),Type::Real) then
                if type(op(ex,[1,1])) = "_power" then
                    return( [op(ex,[1,1,2])] )
                else
                    return( [1] )
                end_if
            end_if
        of "_plus" do
            return(_concat( extract_coeffs(s,x) $ s in ex ))
        otherwise
            if testtype(ex,Type::Real) then
                return( [ex] )
            else
                error("interner Fehler: Ungueltiges Argument")
            end_if
        end_case
    end_proc:

    ex:= split(extract_coeffs(ex,x),x->domtype(x)<>DOM_IDENT);
    if ex[1] = [] then
        return( [],ex[2] )
    else
        return( [max(op(ex[1]))],ex[2] )
    end_if
end_proc:

/*---------------------------------------------------------------------------
    check_on_integer(l <,every>)

    l    : list von Ausdruecken
    every: TRUE oder FALSE (Default: FALSE)
   
    Ist checkonly = FALSE und ist ein Listeneintrag eine natuerliche Zahlen 
    so wird TRUE geliefert.
    Ist checkonyl = TRUE so wird festgestellt, ob alle Listeneintraege 
    natuerlich sind.
---------------------------------------------------------------------------*/

analysis::check_on_integer := proc(nlist,every=FALSE)
    local list;
begin
    list:= map( nlist,x-> bool(is(x,Type::PosInt) = TRUE) );
    if args(0)=1 or not every then
        if nops(list)=1 then return(list[1]) else return(list) end_if
    else
        return( bool(contains( list,FALSE ) = 0) )
    end_if
end_proc:

