alias( genGerade = analysis::gerade ):
alias( Folge = analysis::Folge ):
alias( Funktion = analysis::Funktion ):
alias( FunkTerm(f) = op(f,1) ):
alias( FunkVar(f)  = op(f,2) ):
alias( zuFunktionen(f) = analysis::zuFunktionen(f) ):
alias( zuFolgen(f) = analysis::zuFolgen(f) ):
alias( genTangente = analysis::tangente ):
alias( isIn = analysis::isIn ):

analysis::findZerosInRange := proc(f0, range, genauigkeit = 0.1)
    local temp, f, ende,start, stemp, zeroList, z;
    begin

        if args(0) < 2 then
            error("Erwarte mindestens 2 Parameter")
        end_if;

        f     := op(zuFunktionen(f0));
        range := analysis::deEqualize(range);

        
        start := float(op(range ,1));
        ende := float(op(range, 2));
    
        if start >= ende then
            return([]);
        end:

        temp := start;
        stemp   := (f(temp));

        zeroList := [];

        while temp <= ende do
            if sign(stemp) <> sign(f(temp)) then
                z:= numeric::realroot(FunkTerm(f),FunkVar(f)=start..temp);
                zeroList := append(zeroList, z);
                start := z + genauigkeit;
                temp := z + 2* genauigkeit;
                stemp := (f(start));
            else
                temp := temp + genauigkeit;
            end:
        end_while;
        
        if abs(f(ende)) < _power(10.0, -DIGITS) then
            zeroList := append(zeroList, ende);
        end:

        return(zeroList);
    end_proc:

analysis::zerlegeIntervs := proc(liste, range)
    local i, u, o, intervList;
    begin
        liste := analysis::deEqualize(liste);
        range := analysis::deEqualize(range);

        u := float(op(range ,1));
        o := float(op(range, 2));

        intervList := [];
        for i from 1 to nops(liste) do
            if isIn(u..o, float(liste[i])) then
                if abs(float(u - liste[i])) > _power(10.0, -DIGITS) then
                    intervList := append(intervList, u..float(liste[i]));
                end_if;
                u := float(liste[i]);
            end_if;
        end_for;
        if nops(liste) > 0 and liste[nops(liste)] < o then
            intervList := append(intervList, float(liste[nops(liste)])..o);
        end_if;
        return(intervList);
    end_proc:

analysis::nsBisektion := proc(f, bereich, genau = _power(10, -DIGITS))
    local liste, temp, m, mitte, i, iterativ, iter;
    begin
        if args(0) < 2 then
            error("Erwarte mindestens 2 Parameter")
        end:

        f := op(zuFunktionen(f));

        if nops([f]) <> 1 or domtype(f) <> Funktion then
            error("Erwarte eine Funktion als 1. Parameter");
        end:
        if nops(indets(op(f,1))) > 1 then 
            error("Funktionsterm enthaelt zu viele Unbestimmte");
        end:
        
        bereich := analysis::deEqualize(bereich):
        bereich := float(op(bereich, 1))..float(op(bereich, 2)):
        
        if sign(f(op(bereich, 1))) = sign(f(op(bereich, 2))) then
            error("Funktion muss ueber den Grenzen des Bereiches einen Vorzeichenwechsel haben")
        end:

        if type(bereich) <> "_range" then
            error("Erwarte eine Gleichung als 2. Parameter");
        end:

        iterativ := FALSE;
        if domtype(genau) = DOM_INT and genau > 0 then
            iterativ := TRUE;
            iter := genau;
            genau := _power(10.0, -DIGITS);
        else
            genau := float(analysis::deEqualize(genau));            
            if domtype(genau) <> DOM_FLOAT then
                error("Erwarte einen numerischen Wert oder positive ganze Zahl als 3. Parameter");
            end:
        end:
        
        mitte := proc(m1, m2) begin return((m1 + m2)/2.0) end;
        temp  := [op(bereich)];
        liste := [];

        if iterativ then
            for i from 1 to iter do
                m := mitte(temp[1], temp[2]);

                if sign(f(m)) <> sign(f(temp[1])) then
                    temp[2] := m;
                else
                    temp[1] := m;
                end;

                liste   := append(liste, temp[1]..temp[2]); 
            end_for;
        else
            while abs(temp[1] - temp[2]) >= genau do
                m := mitte(temp[1], temp[2]);

                if sign(f(m)) <> sign(f(temp[1])) then
                    temp[2] := m;
                else
                    temp[1] := m;
                end;

                liste   := append(liste, temp[1]..temp[2]); 
            end_while;
        end;
        return(liste);
    end:
analysis::nsNewton := proc(f, start, genau = _power(10.0, -DIGITS))
    local iter, i, iterativ, liste, temp, fstrich;
    begin
        if args(0) < 2 then
            error("Erwarte mindestens zwei Parameter")
        end:

        f := op(zuFunktionen(f));

        if nops([f]) <> 1 or domtype(f) <> Funktion then
            error("Erwarte eine Funktion im 1. Parameter");
        end:
        if nops(indets(op(f,1)) minus Type::ConstantIdents) > 1 then 
            error("Der Funktionsterm enthaelt zu viele Unbestimmte");
        end:
        
        start   := float(analysis::deEqualize(start)):

        if domtype(start) <> DOM_FLOAT then
            error("Erwarte einen numerischen Wert als 2. Parameter");
        end:

        iterativ := FALSE;
        if domtype(genau) = DOM_INT and genau > 0 then
            iterativ := TRUE;
            iter := genau;
            genau := _power(10.0, -DIGITS);
        else
            genau := float(analysis::deEqualize(genau));            
            if domtype(genau) <> DOM_FLOAT then
                error("Erwarte einen numerischen Wert oder eine positivie ganze Zahl als 3. Parameter");
            end:
        end:

        temp := start;
        liste := [temp];
    
        fstrich := diff(f);

        if not iterativ then
            while abs(float(f(temp))) > genau do
                temp := temp - f(temp)/(fstrich)(temp);
                liste := append(liste, temp);
            end_while;
        else
            for i from 1 to iter-1 do
                temp := temp - f(temp)/(fstrich)(temp);
                liste := append(liste, temp);
            end:
        end;
       return(liste):
    end:
analysis::nsSekantenVerfahren := proc(f, x0, x1, genau = _power(10.0, -DIGITS))
    local iterativ, iter, i, xi, liste, Phi;
    begin
        if args(0) < 3 then
            error("Erwarte mindestens 3 Parameter")
        end:

        f := op(zuFunktionen(f));

        if nops([f]) <> 1 or domtype(f) <> Funktion then
            error("Erwarte eine Funktion als 1. Parameter");
        end:
        if nops(indets(op(f,1))) > 1 then 
            error("Funktionsterm enthaelt zu viele Unbekannte");
        end:

        x0  := float(analysis::deEqualize(x0)):     
        x1  := float(analysis::deEqualize(x1)):

        if domtype(x0) <> DOM_FLOAT then
            error("Erwarte einen numerischen Wert als 2. Parameter");
        end:

        if domtype(x1) <> DOM_FLOAT then
            error("Erwarte einen numerischen Wert als 2. Parameter");
        end:

        iterativ := FALSE;
        if domtype(genau) = DOM_INT and genau > 0 then
            iterativ := TRUE;
            iter := genau;
            genau := _power(10.0, -DIGITS);
        else
            genau := float(analysis::deEqualize(genau));            
            if domtype(genau) <> DOM_FLOAT then
                error("Erwarte einen numerischen Wert oder eine positive ganze Zahl als 3. Parameter");
            end:
        end:


        Phi := proc(f, xi, xi2)
        begin
            return(  (xi2 * f(xi) - xi * f(xi2))/(f(xi) - f(xi2))   );          
        end:

        if float(x0) = float(x1) then 
            error("Die linke und rechte Grenze duerfen nicht uebereinstimmen")
        else
            liste := [];

            if iterativ then
                for i from 1 to iter-1 do
                    liste := append(liste, [x0, x1]);
                    xi := Phi(f, x0 * 1.0, x1 * 1.0);
                    x0 := x1;
                    x1 := xi;
                end:
            else
                while abs(x0 - x1) * 1.0 >  genau do
                    liste := append(liste, [x0, x1]);
                    xi := Phi(f, x0 * 1.0, x1 * 1.0);
                    x0 := x1;
                    x1 := xi;
                end:
            end;
            return(append(liste, [x0, x1]));
        end:
    end:
    
