//********************************************************
// analysis - Eine Library fuer den Analysis Unterricht  *
//                                                       *
// Mirko Igel, Markus Thewes, Sandra Meier-Tokic         *
// Oktober 1999                                          *
//********************************************************

analysis := newDomain( "analysis" ):

analysis::info := "Eine Library fuer den Analysis-Unterricht an Schulen":

analysis::global_term_list:= [
 "tan", "sin", "cos", "cot", "exp", "ln", "sqrt", "arctan", "arcsin", 
 "arccos", "arccot", "sinh", "cosh", "tanh", "arctanh", "arcsinh", "arccosh"
]:

analysis::_GITTERFARBE := RGB::Gray:
analysis::_SCHRAFFURDICHTE := 10:
analysis::_SCHRAFFURFARBE:= RGB::Green:
analysis::_BARFARBE:= RGB::Blue:
analysis::_RICHTUNGSFELDFARBE:= RGB::Black:
analysis::_HILFSFARBE:= RGB::Gray:
analysis::_GRAPHFARBE := RGB::Red:
analysis::_FUELLFARBE := RGB::Yellow:
analysis::_OBERSUMME:= RGB::Gray:
analysis::_UNTERSUMME := RGB::Yellow:
analysis::_RAHMENFARBE:= RGB::Red:
analysis::_DICKE:= 0.3 * unit::mm:
analysis::_GRID:= 150:
analysis::_UMGEBUNGSFARBE:= RGB::Red:
analysis::_POINTSTYLE:= FilledCircles:

analysis::interface:= {
 // Datentypen:
 hold(Funktion), hold(Folge), hold(zusFunktion),

 // Funktionen zur Nullstellenbestimmung:
 hold(nsBisektion), hold(nsNewton), hold(nsSekantenVerfahren),

 // Weitere Hilfsfuktionen:
 hold(ersetzen), hold(wertetabelle), hold(ableitung), hold(knoten),
 hold(monotonie), hold(differenzenQuotient), hold(gerade), hold(tangente),
 hold(istgleich),

 // Grafik:
 hold(graph), hold(genRichtungsfeld), hold(graphSekantenVerfahren), 
 hold(graphBisektion), hold(graphNewton), hold(genUmgebung), hold(graphLimes),

 //symbolische Integration
 hold(integriere), hold(intTip)
}:


/****************************************************************************/
/****************************************************************************/

alias (path = pathname( "ANALYSIS", "DOMAINS" )):

analysis::Funktion := loadproc( analysis::Funktion, path, "Funktion" ):
analysis::Folge := loadproc( analysis::Folge, path, "Folge" ):
analysis::zusFunktion := loadproc( analysis::zusFunktion, path, "zusFunktion" ):

unalias( path ):

/****************************************************************************/

alias (path = pathname( "ANALYSIS", "GRAFIK" )):

analysis::graphSekantenVerfahren:= loadproc( analysis::graphSekantenVerfahren, path, "graphSek"):
analysis::graphBisektion:= loadproc( analysis::graphBisektion, path, "graphBisekt"):
analysis::graphNewton:= loadproc( analysis::graphNewton, path, "graphNewton"):
analysis::graphLimes:= loadproc( analysis::graphLimes, path, "graphLimes" ):

analysis::graph:= loadproc( analysis::graph,path, "graph"):

analysis::genRichtungsfeld:= loadproc( analysis::genRichtungsfeld, path, "genRichtungsfeld" ):
analysis::genUmgebung:= loadproc( analysis::genUmgebung,path, "genUmgebung" ):

unalias( path ):

/****************************************************************************/

alias(path = pathname( "ANALYSIS", "NULLST" )):

analysis::nsBisektion := loadproc( analysis::nsBisektion,path, "verfahren"):
analysis::nsNewton := loadproc( analysis::nsNewton,path, "verfahren"):
analysis::nsSekantenVerfahren := loadproc( analysis::nsSekantenVerfahren, path, "verfahren"):

// Tools
analysis::findZerosInRange:= loadproc( analysis::findZerosInRange, path, "verfahren"):
analysis::zerlegeIntervs:= loadproc( analysis::zerlegeIntervs,path, "verfahren"):

unalias( path ):

/****************************************************************************/

unalias( path ):

/****************************************************************************/

alias(path = pathname( "ANALYSIS", "TOOLS" )):

analysis::differenzenQuotient := loadproc( analysis::differenzenQuotient, path, "analytisch"):
analysis::lim:= loadproc( analysis::lim,path, "analytisch"):
analysis::monotonie := loadproc( analysis::monotonie,path, "monotonie"):
analysis::ableitung := loadproc( analysis::ableitung,path, "ableitung"):
analysis::knoten:= loadproc( analysis::knoten,path, "ableitung"):
analysis::tangenten:= loadproc( analysis::tangenten, path, "tangenten" ):
analysis::sekanten:= loadproc( analysis::sekanten, path, "tangenten" ):
analysis::isIn := loadproc( analysis::isIn, path, "isIn"): 
analysis::zuFunktionen:= loadproc( analysis::zuFunktionen, path, "wandler"): 
analysis::zuFolgen := loadproc( analysis::zuFolgen,path, "wandler"):
analysis::wertetabelle:= loadproc( analysis::wertetabelle,path, "wertetab"):
analysis::gerade:= loadproc( analysis::gerade, path, "gerade"):
analysis::tangente:= loadproc( analysis::tangente, path, "tangente"):
analysis::pairwiseIntersect:= loadproc( analysis::pairwiseIntersect,path, "pairwiseIntersect"):
analysis::deEqualize:= loadproc( analysis::deEqualize,path, "deEqualize"):
analysis::ersetzen:= loadproc( analysis::ersetzen,path, "ersetzen"):
analysis::istPolynom:= (p,x) -> not has( op(rationalize(p),2),x ):
analysis::istgleich:= loadproc( analysis::istgleich, path, "istgleich"):

unalias( path ):

/****************************************************************************/

alias(path = pathname( "ANALYSIS", "INT" )):

analysis::integriere:= loadproc( analysis::integriere, path, "integriere" ):
analysis::intTip:= loadproc( analysis::intTip,path, "intTip" ):
analysis::tip_partielleIntegration:= loadproc( analysis::tip_partielleIntegration, path, "partIntTip" ):

// Hilfsroutinen fuer intTip und integriere
analysis::partialbruchzerlegung:= loadproc( analysis::partialbruchzerlegung, path, "tools" ):
analysis::linear:= loadproc( analysis::linear, path, "tools" ):

unalias( path ):

/****************************************************************************/

alias (path = pathname( "ANALYSIS" )):

analysis::init:= loadproc( analysis::init,path, "init" ):

unalias( path ):

/****************************************************************************/
/****************************************************************************/

