// Mirko Igel
// ueberarbeitet: frankp

alias( Folge       = analysis::Folge ):
alias( Funktion    = analysis::Funktion ):
alias( zusFunktion = analysis::zusFunktion ):
alias( zuFolgen    = analysis::zuFolgen ):
alias( zuFunktionen= analysis::zuFunktionen):
alias( FolgTerm(f) = extop(f, 1) ):
alias( FolgVar(f)  = extop(f, 2) ):
alias( FunkTerm(f) = op(f,1) ):
alias( FunkVar(f)  = op(f,2) ):

Folge := newDomain( "analysis::Folge" ):
Folge::info := "Ein Datentyp fuer reelle Zahlenfolgen":

Folge::new := proc(x, f)
begin
    if args(0) <> 2 then
        error("Erwarte 2 Argument: x und einen Term in x")
    elif domtype(x) <> DOM_IDENT then
        error("Erwarte einen Bezeichner als 1. Argument")
    elif not testtype(f, Type::Arithmetical) then 
        error("Erwarten einen Term vom Typ 'Type::Arithmetical' als 2. Argument")
    elif has( map(indets(f) minus {x} minus Type::ConstantIdents, is, Type::Real), [FALSE, UNKNOWN] ) then 
        error ("Erwarte eine reellwertige Folge");
    else
        assume(x,Type::PosInt,_and):
        return( new(Folge, f, x) )
    end_if
end_proc:

Folge::expr2text := proc(f)
begin
    return(expr2text(FolgVar(f))." +-> ".expr2text(FolgTerm(f)));
end_proc:

Folge::print:= f -> FolgTerm(f):

if version() = [2,0,0] then 
  Folge::Content:= FAIL
else
  Folge::Content := proc(Out, f)
  begin
    Out::Capply(Out::Clambda, Out(FolgVar(f)), Out(FolgTerm(f)))
  end_proc:
end_if:

Folge::expr := f -> FolgTerm(f):

Folge::einsetzen := proc(f,s)
begin
    return( subs(FolgTerm(f),FolgVar(f) = s) )
end_proc:

Folge::ersetzen := proc(f)
begin
    new(Folge, eval(subs(FolgTerm(f), (args(2..args(0))))), FolgVar(f));
end_proc:

Folge::subs := proc(f)
begin
    new(Folge, (subs(FolgTerm(f), (args(2..args(0))))), FolgVar(f));
end_proc:

Folge::func_call := proc(f,s)
begin
    f:= context(f); s:=  context(s);

    if testtype(s,Type::PosInt) or is(s,Type::PosInt) = TRUE then 
        return( eval(subs(FolgTerm(f),FolgVar(f) = s)) );
    else
        error("Folge ist nur fuer natuerliche Werte definiert");
    end_if
end_proc:

Folge::indets := proc(f)
begin
    return(indets(FolgTerm(f)));
end_proc:

Folge::limit := proc(f,x)
begin
    if args(0) > 1 then
        if type(x) <> "_equal" or op(x,2) <> infinity then
            error("Grenzwertbestimmung von Folgen ist nur gegen Unendlich (infinity) moeglich")
        elif op(x,1) <> FolgVar(f) then
            error("Variablen stimmen nicht ueberein")
        end_if
    end_if;
    return(limit(FolgTerm(f), FolgVar(f) = infinity));
end_proc:

Folge::neueLV := proc(f, n)
begin
    return(Folge(n, subs(FolgTerm(f), FolgVar(f) = n)));
end_proc:

Folge::_negate := proc(f)
begin
    return(Folge(FolgVar(f), _negate(FolgTerm(f))));
end_proc:

Folge::_plus := proc(f1,f2)
    local tmp, i;
begin
    case args(0) 
    of 1 do 
        return(f1)
    of 2 do
        if domtype(f1) = Funktion or domtype(f1) = zusFunktion then
            error("Ungueltiger Typ");
        elif domtype(f2) = Funktion or domtype(f2) = zusFunktion then
            error("Ungueltiger Typ");
        elif domtype(f1) <> Folge then
             f1 := Folge::new(FunkVar(f2), f1);
        elif domtype(f2) <> Folge then
             f2 := Folge::new(FunkVar(f1), f2);
        end_if;
        return(Folge::new(FolgVar(f1), _plus(FolgTerm(f1), FolgTerm(f2))));
    otherwise
        tmp := _plus(f1,f2);
        for i from 3 to args(0) do
            tmp := hold(_plus) (tmp + args(i));
        end_for;
        return( eval(tmp) );
    end_case;
end_proc:

Folge::_subtract := proc(f1,f2)
begin
    if domtype(f1) = Funktion or domtype(f1) = zusFunktion then
        error("Ungueltiger Typ");
    elif domtype(f2) = Funktion or domtype(f2) = zusFunktion then
        error("Ungueltiger Typ");
    elif domtype(f1) <> Folge then
        f1 := Folge::new(FunkVar(f2), f1);
    elif domtype(f2) <> Folge then
        f2 := Folge::new(FunkVar(f1), f2);
    end_if;
    return(Folge::new(FolgVar(f1), _subtract(FolgTerm(f1), FolgTerm(f2))))
end_proc:

Folge::_mult := proc(f1,f2)
    local tmp, i;
begin
    if hastype([args()], Funktion) or hastype([args()], zusFunktion) then
        error("Ungueltiger Typ");
    end_if;
      
    case args(0) 
    of 1 do
        return( f1 )
    of 2 do
        if domtype(f1) = Funktion or domtype(f1) = zusFunktion then
            error("Illegaler Typ in Operation");
        elif domtype(f2) = Funktion or domtype(f2) = zusFunktion then
            error("Illegaler Typ in Operation");
        elif domtype(f1) <> Folge then
            f1 := Folge(FunkVar(f2), f1);
        elif domtype(f2) <> Folge then
            f2 := Folge(FunkVar(f1), f2);
        end_if;
        return(Folge::new(FolgVar(f1), _mult(FolgTerm(f1), FolgTerm(f2))))
    otherwise
        tmp := _mult(f1,f2);
        for i from 3 to args(0) do
            tmp := hold(_mult)(tmp * args(i));
        end_for;
        return( eval(tmp) );
    end_case;
end_proc:

Folge::_divide := proc(f1,f2)   
begin                  
    if domtype(f2) = Folge then
        if sign(FolgTerm(f2)) = 0 then
            error("Division durch Null");
        end_if;
    elif sign(f2) = 0 then;
        error("Division durch Null")
    end_if;

    if domtype(f1) = Folge then
        if domtype(f2) <> Folge then
            return(Folge::new(FolgVar(f1), FolgTerm(f1) / f2))
        else
            f2 := Folge::neueLV(f2, FolgVar(f1)):
            return(new(Folge, FolgTerm(f1) / FolgTerm(f2), FolgVar(f1)))
        end_if
    elif domtype(f2) = Folge then
        return(Folge::new(FolgVar(f2), f1 / FolgTerm(f2)));
    else
        return( FAIL )
    end_if
end_proc:

Folge::_power := proc(f, n)
begin
   if contains( {Funktion,Folge,zusFunktion},domtype(n) ) or
       not testtype(n,Type::Arithmetical) 
    then
        error("Ungueltiger Exponent")
    else
        return(new(Folge, FolgTerm(f)^n, FolgVar(f)))
    end_if
end_proc:

Folge::erweitern := proc(f, erw)
    local z, n;
begin
    z:= expand(numer(FolgTerm(f)) * erw);
    n:= expand(denom(FolgTerm(f)) * erw);

    new(Folge, z / n, FolgVar(f));
end_proc:

Folge::_fconcat := proc(f,s)
    local i, tmp;
begin
    case args(0) 
    of 1 do 
        error("Zu wenige Parameter");
    of 2 do
        if domtype(f) = Folge then
            if domtype(s) = Folge then
                f := Folge::neueLV(f, FolgVar(s));
                return(new(Folge, f(FolgTerm(s)), FolgVar(s) ))
            else
                error("Illegale Komposition")
            end_if
        else
            tmp:= f(FolgTerm(s));
            if not testtype( tmp,Type::Arithmetical ) then
                error("Illegale Komposition");
            else
                return(new(Folge, tmp, FolgVar(s) ))
            end_if
        end_if
    otherwise
        tmp := _fconcat(f,s);
        for i from 3 to args(0) do
           tmp := hold(_fconcat) (tmp @ args(i));
        end_for;
        return( eval(tmp) );
    end_case;
end_proc:

Folge::diff := proc(f, var)
begin
    error("Folgen koennen nicht differenziert werden");
end_proc:

Folge::visLim :=
proc(folge, indexbereich, farbe=hold(Color) = hold(Scale))
  local z, bereich, start, ende, sf, list, col, i;
begin
  if args(0) < 2 then
    error("Erwarte mindestens zwei Parameter");
  end:

  folge := op(analysis::zuFolgen(folge));
  
  if nops([folge]) <> 1 or domtype(folge) <> Folge then
    error("Erwarte eine Folge im 1. Parameter");
  end:
  if nops(indets(FolgTerm(folge))) > 1 then
    error("Folgenterm besteht aus zu vielen Unbestimmten")
  end:

  indexbereich := analysis::deEqualize(indexbereich);

  if type(indexbereich) <> "_range" then
    error("Erwarte einen Indexbereich im 2. Parameter")
  end;

  if type(farbe) <> "_equal" or op(farbe,1) <> hold(Color) then
    error("Erwarte 'Color = Liste' (RGB Werte) oder 'Color = Scale'")
  end_if;
  col:= op(farbe,2);
  if col = hold(Scale) then
    col := z -> [0.8 - z, 0.8 - z, 0.8 - z];
  elif not testtype(col, Type::ListOf(Type::Numeric, 3,3)) then
    error("Erwarte 'Color = Liste' (RGB Werte) oder 'Color = Scale'");
  end_if;

  sf := sort([float(folge(i)) $ i = indexbereich]);
  bereich := op(sf, 1)..op(sf, nops(sf));

  z := genident():
  start := op(indexbereich,1);
  ende := op(indexbereich,2);
  list := [];
    
  for i from start to ende do
    sf := float(folge(i));

    if analysis::isIn(bereich, sf) then
      if op(farbe,2) <> hold(Scale) then
        list := append(list,
                       plot::Line2d([sf, -.05],
                                    [sf, .05],
                                    hold(Color) = col));
      else
        list := append(list,
                       plot::Line2d([sf, -.05],
                                    [sf, .05],
                                    hold(Color) =
                                    col(0.8/(ende-start)*(i-start))));
      end_if;
    end_if;
  end_for;

  return(plot::Group2d
         ( op(list),
          plot::PointList2d([[float(op(bereich, 1)), 1],
          [float(op(bereich, 2)), -1]], Visible = FALSE),
          hold(Color) = plot::getDefault(plot::Scene2d::BackGroundColor)))
end_proc:

Folge::genBars :=
proc(folge, bereich)
  local  plist, list, i, temp;
begin
  list := [];
  plist := [];
  folge := op(zuFolgen(folge));

  for i from op(bereich, 1) to op(bereich, 2) do
    temp  := float(folge(i));
    list  := append(list, plot::Line2d([float(i), 0], [float(i), temp],
                                       hold(Color) = analysis::_BARFARBE));
    plist := append(plist, [float(i), float(temp)]);
  end_for:
  if plist <> [] then
    return(plot::Group2d( op(list),
                         plot::PointList2d(plist,
                                           hold(Color) = RGB::Red) ))
  else
    return(plot::Group2d( op(list)) )
  end_if
end_proc:

Folge::graph :=
proc(f,r)
  local i;
begin
  if args(0) < 2 then
    error("Erwarte mindenstens zwei Argumente")
  end_if;

  r := analysis::deEqualize(r);
  
  if type(r) <> "_range" or not testtype(r,Type::PosInt..Type::PosInt) then
    error("Erwarte einen Indexbereich positive ganzer Zahlen im 2. Parameter")
  else
    plot::PointList2d( [[i, f(i)] $ i = r], args(3..args(0)) )
  end_if
end_proc:

Folge::map :=
proc(f)
begin
  return(op(map([f], args(2..args(0)))));
end_proc:

Folge::has := f -> has(FolgTerm(f),args(2..args(0))):

Folge::hastype := f -> hastype(FolgTerm(f),args(2..args(0))):

// undefined slots
Folge::Name               := "analysis::Folge": // damit domtype(f)=Folge liefert.
Folge::domtype            := FAIL:    // wird ueber 'Name' geregelt.
Folge::type               := FAIL:
Folge::evaluate           := FAIL:
Folge::posteval           := FAIL:
Folge::subsex             := FAIL:
Folge::slot               := FAIL:
Folge::undefinedEntries   := FAIL:
Folge::allEntries         := FAIL:
Folge::whichEntry         := FAIL:
Folge::new_extelement     := FAIL:
Folge::nops               := FAIL:
Folge::op                 := FAIL:
Folge::_index             := FAIL:

/************************************************************************
make_slot -- create default slot

make_slot(e,f) creates a slot f for the domain e (here: = Folge) by 
converting the slot to a functional expression; arguments of type 'Folge' 
are converted into expressions before the function f is called. The result 
of function calls of the so created slot is an object of type Function.

NOTE: This MUST be the last method defined for Folge.
************************************************************************/
Folge::args2expr:= proc(x) 
begin 
    if args(0) = 1 then
        if domtype(x) = Folge then
            return(extop(x,1))
        else
            return(x)
        end_if
    else
        return(
            op( map([args()], x -> if domtype(x) = Folge then extop(x,1) else x end) )
        )
    end_if
end_proc:

Folge::make_slot:= proc(e,s) 
begin 
    if domtype(s) = DOM_STRING then
        return(eval(text2expr(s)) @ Folge::args2expr)
    else
        return(FAIL)
    end_if
end_proc:

