alias ( zusFunktion = analysis::zusFunktion ):
alias ( Funktion = analysis::Funktion ):
alias ( Folge = analysis::Folge ):
alias ( FunkTerm(f) = op(f,1) ):
alias ( FunkVar(f)  = op(f,2) ):
alias ( zuFolgen(f) = analysis::zuFolgen(f) ):
alias ( zuFunktionen(f) = analysis::zuFunktionen(f) ):
alias ( genGerade = analysis::gerade ):
alias ( genTangente = analysis::tangente ):
alias ( genLinie = analysis::genLinie ):

//*******************  SekVerf *****************
analysis::graphSekantenVerfahren :=
proc(f, x0, x1, genau = 10.0^(-DIGITS), bereich = FAIL)
  local paar, temp, temp2, ergebnis, sekanten, hilfslinien;

begin
  if args(0) < 3 then
    error("Erwarte mindestens 3 Parameter")
  end_if;

  f := op(zuFunktionen(f));


  if nops([f]) <> 1 or type(f) <> analysis::Funktion then
    error("Erwarte eine Funktion als 1. Parameter");
  end_if;
  if nops(indets(op(f,1))) > 1 then
    error("Funktionsterm enthaelt zu viele Unbestimmte");
  end_if;

  x0      := float(analysis::deEqualize(x0));
  x1      := float(analysis::deEqualize(x1));
  bereich := analysis::deEqualize(bereich);
  genau   := float(analysis::deEqualize(genau));
        
  if type(x0) <> DOM_FLOAT or type(x1) <> DOM_FLOAT then
    error("Erwarte Gleitkommazahlen als 2. und 3. Parameter");
  end_if;
        
  if bereich <> FAIL and type(bereich) <> "_range" then
    error("Erwarte eine Gleichung als 4. Parameter");
  end_if;
        
  if type(genau) <> DOM_FLOAT then
    error("Erwarte eine Gleitkommazahl als 5. Parameter");
  end_if;

  if x0 = x1 then
    error("linke und rechte Grenze duerfen nicht uebereinstimmen")
  else
            
    temp := analysis::nsSekantenVerfahren(f, x0, x1, genau);

    sekanten    := [];
    hilfslinien := [];

    for paar in temp do
      if abs(float(paar[1] - paar[2])) >  abs(float(genau)) then
        sekanten := append(sekanten,
                           genGerade([paar[1], f(paar[1])],
                                     [paar[2], f(paar[2])], op(f,2))
                           );
        hilfslinien := append( hilfslinien,
                              plot::Line2d([paar[1], 0.0],
                                           [paar[1], float(f(paar[1]))],
                                           hold(Color) = analysis::_HILFSFARBE,
                                           hold(LineStyle = Dashed)) );
      end_if;
    end_for;
            
            if bereich = FAIL then
                bereich := (min@op)(map(temp, (op, 1))) ..
                           (max@op)(map(temp, (op, 1)));
            end_if:

  end_if;
  temp2 := analysis::_GRID;
  sysassign( analysis::_GRID, 2 );
  ergebnis := map(sekanten, analysis::graph, bereich,
                  hold(Color) = analysis::_HILFSFARBE);
  sysassign( analysis::_GRID, temp2 );
  return( plot::Group2d( op(ergebnis),
                        op(hilfslinien),
                        analysis::graph(f, bereich,
                                        hold(Color) = analysis::_GRAPHFARBE,
                            //            hold(TitlePosition) = [0.0, 0.0],
                                        hold(Title) = "Numerisch: ".
                                        expr2text(op(temp[nops(temp)], 1))
          )
        ) );
end_proc:

