// Markus Thewes
// ueberarbeitet: frankp, 20.03.2000

alias(ispoly = analysis::istPolynom):

/************************************************************************
    intTip( f )
    
    f: analysis::Funktion, oder arith. Ausdruck in einer Unbekannten

    Finden eines geeigneten Integrationstips fuer die Integration von f.
    Liefert einen Tip zur Integrationsregel / Integrationsverfahren oder
    FAIL.
*************************************************************************/
analysis::intTip := proc(funktion)
    local ex,x,wert;
begin
    if args(0) = 1 then
        funktion := analysis::zuFunktionen(funktion)[1];
        if funktion <> FAIL then  
            ex := op(funktion,1);
            x := op(funktion,2);
        else  
            error("Der 1. Parameter muss ein Term oder eine Funktion in einer Unbekannten sein");
        end_if;
    else
        error("Erwarte genau einen Parameter")
    end_if;

    wert := analysis::tip_grundintegral(ex,x);
    if wert <> FAIL then return(wert) end_if;

    case type(ex)
    of "_plus" do
        return(analysis::tip_summe(ex,x));
    of "_mult" do
        wert := analysis::tip_kuerzen(ex,x);
        if wert <> FAIL then
            return(wert)
        elif denom(ex) <> 1 then
            wert := analysis::tip_substitution(ex,x);
            if wert<> FAIL then  
                return(wert)
            else
                wert := analysis::tip_echtgebrochen(ex,x);
                if wert <> FAIL then
                    return(wert)
                else
                    wert := analysis::tip_polynomdivision(ex,x);
                    if wert <> FAIL then
                        return(wert)
                    else
                        // letzter Versuch, kann FAIL liefern:
                        return( analysis::tipPartInt(ex,x) )
                    end_if
                end_if
            end_if
        else
            wert := analysis::tipPartInt(ex,x);
            if wert <> FAIL then
                return(wert);
            else
                // letzter Versuch, kann FAIL liefern:
                return( analysis::tip_substitution(ex,x) )
            end_if;
        end_if;
    of "_power" do
        wert := analysis::tip_exponential(ex,x);
        if wert <> FAIL then
            return(wert);
        else
            wert := analysis::tipPartInt(ex,x);
            if wert <> FAIL then
                return(wert);
            else
                // letzter Versuch, kann FAIL liefern:
                return( analysis::tip_substitution(ex,x) )
            end_if;
        end_if;
    of "tan" do
        return( analysis::tip_substitution(ex,x) )
    of "sin" do
    of "cos" do
    of "ln" do
    of "arcsin" do
    of "arccos" do
    of "arctan" do
        if op(ex,1) = x then
            return( analysis::tipPartInt(ex,x) )
        else
            return( analysis::tip_substitution(ex,x) )
        end_if;
    of "exp" do
        if op(ex,1) <> x then
            return( analysis::tip_substitution(ex,x) )
        end_if
    end_case;

    return( FAIL )
end_proc:

/*----------------------------------------------------------------------
    tip_substitution(f,x)

    f: arith. Ausdruck
    x: DOM_IDENT

    Finden einer Substitutionsvorschrift. Liefert Substitutionsvorschrift 
    oder FAIL, wenn keine Substitutionsvorschrift gefunden werden kann.
-----------------------------------------------------------------------*/
analysis::tip_substitution := proc(ex,x)
    local wert;
begin
    wert := analysis::lineare_substitution(ex,x);
    if wert <> FAIL then
        return(wert);
    else
        // letzter Versuch, kann FAIL liefern:
        return( analysis::suche_substitution(ex,x) )
    end_if;
end_proc:

/*----------------------------------------------------------------------
    lineare_substitution(f,x)
                     
    f: arith. Ausdruck
    x: DOM_IDENT
                  
    Auffinden einer linearen Substitution. Liefert Aufbau der 
    Baum-Liste fuer die lineare Substitution.
----------------------------------------------------------------------*/
analysis::lineare_substitution := proc(ex,x)
    local arg,create_tree,basis,exponent;
begin
    create_tree := proc(ex,x,arg)
        local z, xs, zs;
    begin
        z:= genident("z");
        xs:= expr2text(x);
        zs:= expr2text(z);
        adt::Tree([
            hold(int)(ex,x), "Integration durch Substitution",
            "int( f(a*".xs." + b) ) d".xs." = int( 1/a*f(".zs.") ) d".zs,
            [z=arg,"Substitutionsgleichung"]
        ])
    end_proc:

    case type(ex) 
    of "sin"    do  
    of "cos"    do  
    of "ln"     do
    of "exp"    do  
    of "arcsin" do
    of "arccos" do
    of "arctan" do
        arg := op(ex,1);
        if analysis::linear(arg,x) then
            return(create_tree(ex,x,arg));
        else
            return(FAIL)
        end_if
    of "tan" do
        arg := op(ex);
        if analysis::linear(arg,x) then
            return(create_tree(ex,x,cos(arg)));
        else
            return(FAIL)
        end_if;
    of "_power" do
        basis := op(ex,1);
        exponent := op(ex,2);
        arg:= type(basis);
        if ispoly(basis,x) and analysis::linear(basis,x) and testtype(exponent,Type::Real) then
            return(create_tree(ex,x,basis));
        elif arg = "sin" or arg = "cos" then
            if analysis::linear(op(basis),x) then
                return(create_tree(ex,x,op(basis)));
            else
                return(FAIL)
            end_if;
        else
            return(FAIL)
        end_if;
    otherwise
        return(FAIL)
    end_case
end_proc:

/*----------------------------------------------------------------------
    suche_substitution(f,x)
    f: arith. Ausdruck
    x: DOM_IDENT

    Finden einer geeigneten Substitution. Liefert Substitutionsvorschrift 
    oder FAIL, wenn keine Substitutionsvorschrift gefunden werden kann.
----------------------------------------------------------------------*/
analysis::suche_substitution := proc(ex,x)
    local faktor1,faktor2,poly_poly_mult,wert,poly_other_mult,other_other_mult,
          zaehler,nenner, arctan_trick,quotienten, create_tree, arcsin_trick,
          koeff,groessen_fuer_arc_trick;
begin
    create_tree := proc(ex,x,subfkt)
        local z, zs, xs;
    begin
        z := genident("z");
        zs := expr2text(z);
        xs := expr2text(x);
        adt::Tree([
            hold(int)(ex,x),[
            "Integration durch Substitution",
            "int( f(g(".xs.")) * g'(".xs.") )d".xs." = int( f(".zs.") )d".zs,
            [z = subfkt,"Substitutionsgleichung"]
        ]])
    end_proc:
         
    poly_poly_mult := proc(faktor1,faktor2,x)
        local basis1,exponent1,basis2,exponent2,dbasis1,dbasis2,quotient;
    begin
        case type(faktor1)
        of "_power" do
            if op(faktor1,1)=x then
                basis1 := faktor1;
                exponent1 := 1;
            else
                basis1 := op(faktor1,1);
                exponent1 := op(faktor1,2);
            end_if;
            break;
        of DOM_IDENT do
        of "_plus" do
            basis1 := faktor1;
            exponent1 := 1;
            break;
        otherwise
            return(FAIL)
        end_case;   

        case type(faktor2)
        of "_power" do
            if op(faktor2,1)=x then
                basis2 := faktor2;
                exponent2 := 1;
            else
                basis2 := op(faktor2,1);
                exponent2 := op(faktor2,2);
            end_if;
            break;
        of DOM_IDENT do
        of "_plus" do
            basis2 := faktor2;
            exponent2 := 1;
            break;
        otherwise
            return(FAIL)
        end_case;   
         
        dbasis1 := diff(basis1,x);
        dbasis2 := diff(basis2,x);
        quotient := simplify(dbasis2/basis1);
        if testtype(quotient,Type::Real) then
            return(create_tree(faktor1*faktor2,x,basis2)); 
        end_if;
        
        quotient := simplify(dbasis1/basis2);
        if  testtype(quotient,Type::Real) then
            return(create_tree(faktor1*faktor2,x,basis1)); 
        end_if;
 
        return(FAIL)
    end_proc:

    poly_other_mult := proc(faktor_poly,faktor_other,x)
        local basis,exponent,argument,dargument,quotient,liste;
    begin
        exponent := 0;
        case type(faktor_other)
        of "sin"    do
        of "cos"    do
        of "ln"     do
        of "exp"    do
            argument := op(faktor_other);
            if type(argument)="_power" then
                exponent := op(argument,2);
            end_if;
            dargument := diff(argument,x);                        
            break;
        of "_power" do
            basis := op(faktor_other,1);
            exponent := op(faktor_other,2);
            if  testtype(basis,Type::Real) then
                argument := exponent;
            elif basis=x then
                argument := faktor_other;
            else
                argument := basis;
            end_if;
            dargument := diff(argument,x);                        
            break;
        otherwise
           return(FAIL);
        end_case;

        quotient := simplify(dargument/faktor_poly);
        if  testtype(quotient,Type::Real) or degree(poly(argument,[x]))= degree(poly(faktor_poly,[x])) or exponent<>0 then
            return(create_tree(faktor_poly*faktor_other,x,argument));
        end_if;
         
        return(FAIL)
    end_proc:
         
    other_other_mult := proc(faktor1,faktor2,x)
        local term1,term2,argument,dargument,ana_faktor1,ana_faktor2;
    begin
        term1 := 1;
        ana_faktor1 := type(faktor1);
        ana_faktor2 := type(faktor2);
        if ana_faktor1="sin" or ana_faktor1 ="cos" then
            term1 := faktor1;
        elif ana_faktor2="sin" or ana_faktor2="cos" then
            term1 := faktor2;
        else
            return(FAIL)
        end_if;

        if term1 <> 1 then
            term2 := simplify((faktor1*faktor2)/term1);
        else
            return(FAIL)
        end_if;
         
        argument := op(term2);
        dargument := diff(argument,x);
        if testtype(simplify(dargument/term1),Type::Real) then
            return(create_tree(faktor1*faktor2,x,argument));  
        else
            return(FAIL)
        end_if;
    end_proc:
 
    koeff := proc(p,x)
        local i;
    begin
        p:= poly(p,[x]);
        [coeff(p,i) $ i=0..degree(p)]
    end_proc:

    groessen_fuer_arc_trick := proc(ex,x)
        local koeffizienten,a,b,c,epsilon,delta,gamma;
    begin
        koeffizienten := koeff(ex,x);
        a := koeffizienten[1];
        b := koeffizienten[2];
        c := koeffizienten[3];
        epsilon := b/a;
        delta := c/a;
        gamma := delta - (epsilon/2)^2;
        return([epsilon,gamma]);
    end_proc:
 
    arctan_trick := proc(zaehler,nenner,x)
       local epsilon,gamma,wert;
    begin
        if testtype(zaehler,Type::Real) and ispoly(nenner,x) and degree(poly(nenner,[x]))=2 then
            wert := groessen_fuer_arc_trick(nenner,x);
            epsilon := wert[1];
            gamma := wert[2];
            if gamma>0 then
                return(create_tree(zaehler/nenner,x,(x + (epsilon/2))/sqrt(gamma)));
            else
                return(FAIL)
            end_if;
        else
            return(FAIL)
        end_if
    end_proc:

    arcsin_trick := proc(zaehler,nenner,x)
        local basis,exponent,epsilon,gamma,wert;
    begin
        if testtype(zaehler,Type::Real) and type(nenner)="_power" then
            basis := op(nenner,1);
            exponent := op(nenner,2);
            if exponent = 1/2 and ispoly(basis,x) and degree(poly(basis,[x])) = 2 then
                wert := groessen_fuer_arc_trick(basis,x);
                epsilon := wert[1];
                gamma := wert[2];
                if gamma < 0 then
                    return(create_tree(zaehler/nenner,x,(x + (epsilon/2))/sqrt(abs(gamma)))); 
                else
                    return(FAIL)
                end_if;
            else
                return(FAIL)
            end_if;                 
        else
            return(FAIL);
        end_if;                                
    end_proc:

    quotienten := proc(zaehler,nenner,x)
        local exponent,basis,argument,faktor1,faktor2; 
    begin
        if testtype(simplify(diff(nenner,x)/zaehler),Type::Real) then
            return(create_tree(zaehler/nenner,x,nenner))
        elif nops(factor(simplify(diff(nenner,x)/zaehler))) = 1 then
            if testtype(op(factor(simplify(diff(nenner,x)/zaehler))),Type::Real) then
                return(create_tree(zaehler/nenner,x,nenner))
            end_if
        end_if;
        if testtype(simplify(diff(zaehler,x)*nenner),Type::Real) and simplify(diff(zaehler,x)*nenner)<>0 then
            return(create_tree(zaehler/nenner,x,zaehler))
        end_if;
            
        if type(nenner)="_power" then
            exponent := op(nenner,2);
            if denom(exponent) <> 1 then
                basis := op(nenner,1)^op(exponent,1)
            else
                basis := op(nenner,1)
            end_if;

            // f'(x)/ (f(x))^(1/n)
            if testtype(simplify(diff(basis,x)/zaehler),Type::Real) then
                return(create_tree(zaehler/nenner,x,basis));  
            end_if;
                     
            // grad f(x) = grad(zaehler)   nenner = basis^exponent, basis = f(x)
            if ( degree(poly(basis,[x]))=degree(poly(zaehler,[x])) )and (basis<>x) then
                return(create_tree(zaehler/nenner,x,basis))
            end_if;

            // 1 + grad f(x) = grad(zaehler) nenner = basis^exponent, basis = f(x)
            if degree(poly(basis,[x]))+1 = degree(poly(zaehler,[x])) and denom(exponent) = 1 then
                return(create_tree(zaehler/nenner,x,nenner))
            end_if
        end_if;
            
        if type(zaehler)="_power" then
            exponent := op(zaehler,2);
            if denom(exponent) <> 1 then
                basis := op(zaehler,1)^op(exponent,1)
            else
                basis := op(zaehler,1)
            end_if;

            if testtype(simplify(diff(basis,x)/nenner),Type::Real) then
                return(create_tree(zaehler/nenner,x,basis))
            end_if;

            if ( degree(poly(basis,[x]))=degree(poly(zaehler,[x])) ) then
                return(create_tree(zaehler/nenner,x,basis))
            end_if;

            // 1 + grad f(x) = grad(zaehler) nenner = basis^exponent, basis = f(x)
            if degree(poly(basis,[x])) = degree(poly(zaehler,[x]))+1 and denom(exponent) <> 1 then
                return(create_tree(zaehler/nenner,x,zaehler)); 
            end_if;              
        end_if;

        case type(zaehler)
        of "exp" do
        of "cos" do
        of "sin" do
        of "ln"  do
            argument := op(zaehler);
            if testtype(diff(argument,x)*nenner,Type::Real) then
                return(create_tree(zaehler/nenner,x,argument))
            else
                return(FAIL)
            end_if;     
        end_case;                                       

        if ( degree(poly(zaehler,[x])) = degree(poly(nenner,[x])) ) and degree(poly(zaehler,[x]))=1 then
            if zaehler = x then
                return(create_tree(zaehler/nenner,x,nenner));  
            else
                return(create_tree(zaehler/nenner,x,zaehler)); 
            end_if;
        end_if;
         
        return(FAIL)
    end_proc:

    nenner := denom(ex);
    if nenner <> 1 then
        zaehler := numer(ex);
        wert := quotienten(zaehler,nenner,x);
        if wert <> FAIL then
            return(wert);
        else
            wert := arctan_trick(zaehler,nenner,x);
            if wert<> FAIL then
                return(wert);
            else
                // letzter Versuch, kann FAIL liefern:
                return( arcsin_trick(zaehler,nenner,x) )
            end_if;
        end_if;

        return(FAIL);                       
    elif type(ex) = "_power" then
        faktor1 := op(ex,1);
        faktor2 := op(ex,2);
        if ispoly(faktor1,x) and ispoly(faktor2,x) then
            return( poly_poly_mult(faktor1,faktor2,x) )
        elif ispoly(faktor1,x) then
            return( poly_other_mult(faktor1,faktor2,x) )
        elif ispoly(faktor2,x) then
            return( poly_other_mult(faktor2,faktor1,x) )
        else
            return( other_other_mult(faktor1,faktor2,x) )
        end_if
    end_if;

    return(FAIL);
end_proc:


/*----------------------------------------------------------------------
    tip_exponential(f,x)

    f: arith. Ausdruck
    x: DOM_IDENT
                  
    Umformen einer Potenzfunktion in eine exp-Funktion.
----------------------------------------------------------------------*/        
analysis::tip_exponential := proc(ex,x)
    local xs, basis, exponent;
begin
    basis := op(ex,1);
    exponent := op(ex,2);
    if testtype(basis,Type::Real) or ispoly(basis,x) and ispoly(exponent,x) and not testtype(exponent,Type::Real) then
        xs := expr2text(x);
        return( adt::Tree( [
            hold(int)(ex,x),
            "Integration einer Potenzfunktion",
            "f(".xs.")= a^( b*".expr2text(exponent)." ) = exp( ".expr2text(exponent)."*b*ln(a) )",
            [exp(exponent*ln(basis)),"= exp( ".expr2text(exponent)." * b * ln(a) )"]
        ]) )
    else
        return(FAIL)
    end_if
end_proc:

/*----------------------------------------------------------------------
    tipPartInt(f,x)

    f: Ausdruck
    x: DOM_IDENT
    
    Suchen nach einer passenden Wahl fuer die parietlle Integration.
    Liefert Baum fuer die partielle Integration.
----------------------------------------------------------------------*/      
analysis::tipPartInt := proc(ex,x)
    local wert,uvStrich_liste,exponenten_liste,besondere_anweisung,anzahl_liste,
          _uv_anzahl,_expon_anzahl,_bes_anzahl,tree,tree2,_x;
begin
    wert := analysis::tip_partielleIntegration(ex,x);
    if wert=FAIL then
        return(FAIL)
    end_if;

    anzahl_liste := nops(wert);
    uvStrich_liste := op(wert,1);
    exponenten_liste := op(wert,2);
    besondere_anweisung := op(wert,3);

    _uv_anzahl := nops(uvStrich_liste);
    _expon_anzahl := nops(exponenten_liste);
    _bes_anzahl := nops(besondere_anweisung);

    _x := expr2text(x);
    if (_uv_anzahl=0) and (_expon_anzahl=1) and (_bes_anzahl<>0) then
        if besondere_anweisung="property" then
            error("Erwarte positiv ganzzahlige Exponenten")
        end_if;
    end_if;

    tree := ["partielle Integration","int(u v') d"._x." = u * v - int(u' v) d"._x];
    if _uv_anzahl=2 and _expon_anzahl >= 0 and _expon_anzahl <= 1 and _bes_anzahl=0 then
        tree := append(tree,
            [uvStrich_liste[1],"= u"],
            [uvStrich_liste[2],"= v'"]
        );
        return(adt::Tree([hold(int)(ex,x),tree]));
    end_if;

    if _uv_anzahl = 2 and _expon_anzahl = 2 and besondere_anweisung = "trig" then
        tree2 := tree;

        // 1. Moeglichkeit:
        tree := append(tree,
            [uvStrich_liste[1],"= u   mit ".op(exponenten_liste,1)." > ".op(exponenten_liste,2)],
            [uvStrich_liste[2],"= v'"]
        );

        // 2. Moeglichkeit:
        tree2 := append(tree2,
            [uvStrich_liste[2],"= u   mit ".op(exponenten_liste,1)." <= ".op(exponenten_liste,2)],
            [uvStrich_liste[1],"=v'"]
        );
        return(adt::Tree([hold(int)(ex,x),tree]),adt::Tree([hold(int)(ex,x),tree2]))
    end_if;

    return(FAIL)
end_proc:

/*----------------------------------------------------------------------
    tip_echtgebrochen(f,x)

    f: arith. Ausdruck
    x: DOM_IDENT

    Suchen nach Partialbruchzerlegung. Liefert Baum fuer die 
    echtgebrochenen Funktionen.
----------------------------------------------------------------------*/      

analysis::tip_echtgebrochen := proc(ex,x)
    local z,n,echte_zerlegung,tree,anzahl_zerlegung,i,xs;
begin
    z := numer(ex);
    n := denom(ex);
    if ispoly(z,x) and ispoly(n,x) and degree(poly(z,[x]))< degree(poly(n,[x])) then
        echte_zerlegung := analysis::partialbruchzerlegung(ex,x)[2];
        if type(echte_zerlegung)="_plus" then
            anzahl_zerlegung := nops(echte_zerlegung);
            if anzahl_zerlegung = 1 then // wirklich notwendig?
                return( FAIL )
            end_if
        else
            return( FAIL )
        end_if;
      
        xs := expr2text(x);
        if anzahl_zerlegung = 2 then
            tree := [
                "Integration durch Partialbruchzerlegung",
                "int( p(".xs.") / q(".xs.") )d".xs." = int( c1/q1 )d".xs." + int( c2/q2 )d".xs
            ]
         else
            tree := [
                "Integration durch Partialbruchzerlegung",
                "int( p(".xs.") / q(".xs.") )d".xs." = int( c1/q1 )d".xs." + ... + int( c".expr2text(anzahl_zerlegung)." / q".expr2text(anzahl_zerlegung)." )d".xs
            ]
         end_if;

         for i from 1 to anzahl_zerlegung do
            tree := append(tree,
                [op(echte_zerlegung,i),"= c".i." / q".i]
            );
         end_for;
         return( adt::Tree([hold(int)(ex,x),tree]) )
    else
        return(FAIL)
    end_if
end_proc:

/*----------------------------------------------------------------------
    tip_polynomdivision(f,x)

    f: arith. Ausdruck
    x: DOM_IDENT
              
    Ergebnis ist ein Baum fuer die Polynomdivision.
----------------------------------------------------------------------*/            
analysis::tip_polynomdivision := proc(ex,x)
    local z,n,nicht_gebrochen,echte_zerlegung,xs,wert;
begin
    z := numer(ex);
    n := denom(ex);
    if ispoly(z,x) and ispoly(n,x) and degree(poly(z,[x])) >= degree(poly(n,[x])) then
        wert := analysis::partialbruchzerlegung(ex,x);
        nicht_gebrochen := wert[1];
        echte_zerlegung := wert[3];

        xs := expr2text(x);
        return(adt::Tree( [hold(int)(ex,x),[
           "Integration durch Polynomdivision",
           "int( p(".xs.") / q(".xs.") )d".xs." = int( g(".xs.") )d".xs." + int( r(".xs.") / q(".xs.") )d".xs,
           [nicht_gebrochen,"= g(".xs.")"],
           [echte_zerlegung,"= r(".xs.") / q(".xs.")"]
        ]] ))
    else
        return(FAIL)
    end_if
end_proc:

/*----------------------------------------------------------------------
    tip_polynomdivision(f,x)

    f: arith. Ausdruck
    x: DOM_IDENT
              
    Ergebnis ist ein Baum fuer Grundintegrale.
----------------------------------------------------------------------*/            
analysis::gundintegrale:= [
    x->cos(x),x->sin(x),x->1/(cos(x)^2),x->1/(sin(x)^2),
    x->exp(x),x->1/x, x->1/(x^2+1), x->1/sqrt(1-x^2)
]:
analysis::tip_grundintegral := proc(ex,x)
    local g,create_tree,basis,exponent;
begin
    create_tree := (f,x) -> adt::Tree([hold(int)(f,x),["Grundintegral"]]):

    if not has(ex,x) or diff(ex,x) = 1 then
        return(create_tree(ex,x))
    end_if;

    case type(ex)
    of "_power" do
        basis := op(ex,1);
        exponent := op(ex,2);
        if basis = x and (testtype(exponent,Type::Real) or not has(exponent,x)) then
            return(create_tree(ex,x))
        end_if;
        break;
    of "_plus" do
        return(FAIL)
    of DOM_COMPLEX do
        return(FAIL)
    end_case;
    if ispoly(ex,x) then return(FAIL) end_if;

    for g in analysis::gundintegrale do
        if analysis::istgleich(g(x),ex,x,hold(Plain)) = TRUE then
            return(create_tree(ex,x))
        end_if
    end_for;

    return(FAIL)
end_proc:

/*----------------------------------------------------------------------
    tip_summe(f,x)

    f: arith. Ausdruck
    x: DOM_IDENT
              
    Zerlegung des Ausdruckes in die einzelnen Summanden. Rueckgabe
    ist ein Baum fuer die Summenregel.
----------------------------------------------------------------------*/            
analysis::tip_summe := proc(ex,x)
    local intern,tree,i,n,xs;
begin
    n := nops(ex);
    xs := expr2text(x);
    intern := 0;
    if n = 2 then
        tree := [
            "Summenregel",
            "int( S1 + S2 ) d".xs." = int( S1 )d".xs." + int( S2 )d".xs
        ]
    else
        tree := [
            "Summenregel",
            "int( S1 + ... + S".n." )d".xs." = int( S1 )d".xs." + ... + int( S".n." )d".xs
        ]
    end_if;

    for i from 1 to n do
        tree := append(tree,[op(ex,i),"= S".i]);
    end_for;
    return( adt::Tree([hold(int)(ex,x),tree]) )
end_proc:

/*----------------------------------------------------------------------
    tip_kuerzen(f,x)

    f: arith. Ausdruck
    x: DOM_IDENT
              
    Vollstaendiges Kuerzen des Ausdruckes. Baum fuer vollstaendig 
    gekuerzten Ausdruck.
----------------------------------------------------------------------*/            
analysis::tip_kuerzen := proc(ex,x)
    local skalarKuerzen,polynomKuerzen,zaehler,nenner;
begin
    polynomKuerzen := proc(ex,x)
        local _x,polynom_fkt,txt,anzahl_gekuerzt,tree,i;
    begin          
        polynom_fkt := analysis::polynom_raus(ex,x);
                  
        if nops(polynom_fkt) <> 1 and nops(polynom_fkt[1]) <> 0 then
            anzahl_gekuerzt := nops(polynom_fkt[1]);
            _x := expr2text(x);
            if denom(polynom_fkt[2]) <> 1 then
                txt := "f("._x.") / g("._x.")";
            else
                txt := "h("._x.")";
            end_if;
            if anzahl_gekuerzt = 1 then
                tree := [
                    "Polynom p("._x.") kuerzen",
                    "int( ( p("._x.") * f("._x.") ) / ( p("._x.") * g("._x.") ) ) d"._x." = int( ".txt." )d"._x
                ];
            elif anzahl_gekuerzt = 2 then
                tree := [
                    "Polynom p("._x.") kuerzen",
                    "int( ( (p1*p2)("._x.") * f("._x.") ) / ( (p1*p2)("._x.") * g("._x.") ) ) d"._x." = int( ".txt." )d"._x
                ];
            else
                tree := [
                    "Polynome p1("._x."),...,p".anzahl_gekuerzt."("._x.") kuerzen",
                    "int( ( (p1*...*p".expr2text(anzahl_gekuerzt).")("._x.") * f("._x.") ) / ( (p1*...*p".expr2text(anzahl_gekuerzt).")("._x.") * g("._x.") ) )d"._x." = int( ".txt." )d"._x
                ];
            end_if;
                           
            if anzahl_gekuerzt=1 then
                tree := append(tree,[polynom_fkt[1][1],"= p"]);
            else
                for i from 1 to anzahl_gekuerzt do
                    tree := append(tree,[polynom_fkt[1][i],"= p".expr2text(i)."("._x.")"]);
                end_for;
            end_if;

            tree := append(tree,[polynom_fkt[2],txt]);
            return(adt::Tree( [hold(int)(ex,x),tree] ))
        else
            return(FAIL)
        end_if
    end_proc:

    skalarKuerzen := proc(ex,x)
        local tree,_x,skalar_fkt;
    begin
        skalar_fkt := analysis::skalar_raus(ex,x);
        if skalar_fkt[1] <> 1 and skalar_fkt[3] = FALSE
           or (skalar_fkt[1] = 1 and skalar_fkt[3])
        then
            _x := expr2text(x);
            if skalar_fkt[3] then
                if testtype(skalar_fkt[4],Type::Real) then
                    tree := [
                        "Skalar s gekuerzt",
                        "int( ( s * f("._x.") ) / ( s * g("._x.") ) )d"._x." = int( f("._x.") / g("._x.") )d"._x.", s aus R"
                    ];
                else
                    tree := [
                        "Faktor s gekuerzt",
                        "int( ( s * f("._x.") ) / ( s * g("._x.") ) )d"._x." = int( f("._x.") / g("._x.") )d"._x
                    ]
                end_if;
                tree := append(tree,
                    [skalar_fkt[4],"= s"],
                    [skalar_fkt[2],"= f("._x.")"]
                );
            else
                tree := [
                    "Faktorregel",
                    "int( s * f("._x.") )d"._x." = s * int( f("._x.") )d"._x.", s aus R",
                    [skalar_fkt[1],"= s"],
                    [skalar_fkt[2],"= f("._x.")"]
                ]
            end_if;
            return(adt::Tree( [hold(int)(ex,x),tree] ))
        else
            return(FAIL)
        end_if
    end_proc;

    nenner := denom(ex);
    if nenner <> 1 then
        zaehler := numer(ex);
        if ispoly(zaehler,x) and ispoly(nenner,x)
           and not testtype(zaehler,Type::Real) and not testtype(nenner,Type::Real) then
            return(polynomKuerzen(ex,x));       
        end_if;
    end_if;

    return(skalarKuerzen(ex,x));
end_proc:

/*------------------------------------------------------------------------------
    skalar_raus(f,x)

    f: arithm. Ausdruck
    x: DOM_IDENT

    Ermittelt saemtliche Skalare/Idents in dem Ausdruck
    Liefert Skalar, die Restfunktion, skalarer Faktor zu 1 gekuerzt? (BOOLEAN), 
    skalarer Faktor, falls skalarer Faktor zu 1 gekuerzt   
------------------------------------------------------------------------------*/
analysis::skalar_raus := proc(ex,x)
    local skalar,rest,liste,i,j,faktor,exponent,vor_ex,fall_1;
begin
    fall_1 := proc(vor_ex,rest)
        local lauf,zaehler_liste,skalar;
    begin
        zaehler_liste := maprat( numer(vor_ex)/numer(rest),factor );
        if nops(zaehler_liste) = 1 then
            zaehler_liste:= zaehler_liste[1];
            if domtype(zaehler_liste) = DOM_IDENT or testtype(zaehler_liste,Type::Real ) then
                return(zaehler_liste)
            end_if
        else
            skalar := 1;
            for lauf from 2 to nops(zaehler_liste) step 2 do
               skalar := skalar * (zaehler_liste[lauf])^(zaehler_liste[lauf+1]);
            end_for;
            return(skalar)
        end_if
    end_proc:

    vor_ex := ex;
    ex := normal(ex);
    if traperror(maprat(ex,factor)) <> 0 then 
        return([1,ex,FALSE])
    end_if;

    liste := maprat(ex,factor);
    skalar := liste[1];
    rest := 1;
    for i from 2 to nops(liste) step 2 do
        if traperror(factor(liste[i])) <> 0 then return([1,ex,FALSE]) end_if;
        faktor := factor(liste[i]);
        exponent := liste[i+1];
        if nops(faktor)=1 then
            if has(indets(faktor),x) then
                rest := rest*op(faktor)^exponent;
                skalar := skalar*op(faktor)^exponent;
            end_if
        else
            for j from 2 to nops(faktor) step 2 do
                if has(indets(faktor[j]),x) then
                    rest := rest* (faktor[j]^(faktor[j+1]))^exponent;
                else
                    skalar := skalar*(faktor[j]^(faktor[j+1]))^exponent;
                end_if
            end_for
        end_if
    end_for; 

    rest := normal(rest);
    if analysis::istgleich(skalar,1,x,Plain) and not analysis::istgleich(numer(vor_ex),numer(rest),x,Plain)
       and not analysis::istgleich(denom(vor_ex),denom(rest),x,Plain)
    then
        return([skalar,rest,TRUE,fall_1(vor_ex,rest)])
    else
        return([skalar,rest,FALSE])
    end_if
end_proc:

