// Markus Thewes
// ueberarbeitet: frankp, 20.03.2000

alias( ispoly = analysis::istPolynom ):

/************************************************************************
    istgleich( f,g <,Plain> )
    istgleich( f=g <,Plain> )
    
    f,g  : analysis::Funktionen oder arith. Ausdruecke in einer Unbekannten
    Plain: DOM_IDENT

    Liefert TRUE, wenn g und f gleich sind, sonst FALSE oder UNKNOWN,
    falls es nicht entschieden werden kann.
*************************************************************************/
analysis::istgleich := proc()
    local f1, f2, x, a, term, vereinfache, umformen, light;
begin
     vereinfache := proc(term1, term2)
         local differenz;
     begin
         differenz := term1-term2;
         if not iszero(differenz) then
            differenz := umformen(differenz);
            if not iszero(differenz) then
                differenz := normal(differenz);
                if not iszero(differenz) then
                    differenz := simplify(differenz);
                    if not iszero(differenz) then
                        differenz := expand(differenz);
                    end_if
                end_if
            end_if
         end_if;
         return(differenz)
     end_proc:
          
     umformen := proc(ausdruck)
     begin
        if has(ausdruck,sin) and has(ausdruck,cos) then
            ausdruck := combine(ausdruck,sincos);
        end_if;

        if has(ausdruck,arctan) then
            ausdruck := combine(ausdruck,arctan);
        end_if;

        if has(ausdruck,sin) or has(ausdruck,cos) and has(ausdruck,tan) then
            ausdruck := rewrite(ausdruck,sincos);
        end_if;

        if has(ausdruck,tan) then
            ausdruck := rewrite(ausdruck,sincos);
        end_if;

        if has(ausdruck,exp) then
            ausdruck := combine(ausdruck,exp);
        end_if;

        if has(ausdruck,ln) then
            ausdruck := combine(ausdruck,ln);
        end_if;

        if  has(ausdruck,sinh)and has(ausdruck,cosh) then
            ausdruck := combine(ausdruck,sinhcosh);
        end_if;

        return(ausdruck);
    end_proc:

    if args(0) = 0 then error("Keine Parameter uebergeben") end_if;
    a:= [args()];
    if (x:= contains(a,hold(Plain))) <> 0 then
        light:= TRUE;
        delete a[x]
    else
        light:= FALSE
    end_if;

    case nops(a)
    of 0 do
        error("Zuwenige Parameter uebergeben")
    of 1 do
        if type(a[1])="_equal" then
            f2:= op(a[1],2);
            f1:= op(a[1],1);
            x:= NIL;
            break
        else
            error("Erwarte eine Gleichung fuer den 1. Parameter")
        end_if
    of 2 do
        if type(a[1])="_equal" then
            f2:= op(a[1],2);
            f1:= op(a[2],1);
            x:= a[2]
        else
            f1:= a[1]; f2:= a[2];
            x:= NIL
        end_if;
        break
    of 3 do
        ([f1,f2,x]):= a;
        break
    otherwise   
        error("Zuviele Parameter")
    end_case;

    if domtype(f1) = analysis::Funktion then
        if domtype(f2) <> analysis::Funktion then
            if x = NIL then 
                x:= op(f1,2);
                f2:= analysis::Funktion(x,f2); // Zum Testen
            elif x <> op(f1,2) then
                error("Variablen stimmen nicht ueberein")
            end_if
        elif op(f1,2) <> op(f2,2) then
            error("Variablen stimmen nicht ueberein")
        end_if;

        f1:= op(f1,1);
        f2:= op(f2,1);
    elif domtype(f2) = analysis::Funktion then
        if x = NIL then 
            x:= op(f2,2);
            f1:= analysis::Funktion(x,f1); // Zum Testen
        elif x <> op(f2,2) then
            error("Variablen stimmen nicht ueberein")
        end_if;

        f1:= op(f1,1);
        f2:= op(f2,1);
    elif x = NIL then
        x:= indets(f1) union indets(f2) minus Type::ConstantIdents;
        if nops(x) <> 1 then
            error("Zuviele Parameter oder unterschiedlichen Variablen")
        else
            x:= op(x,1)
        end_if
    end_if;

    term := vereinfache(f1, f2);
    if iszero(term) then
        return(TRUE)
    else
        case type(term)
        of DOM_COMPLEX do
            return(FALSE)
        of DOM_IDENT do
            return( is(term = 0) )
        otherwise
            if testtype(term,Type::Real) then
                if abs(term) < 10^(-DIGITS) then
                    return(TRUE)
                else
                    return(FALSE)
                end_if
            elif ispoly(term,x) then
                return(FALSE)
            elif light then
                return(UNKNOWN)
            else
                term:= int(term,x);
                if type(term) = "int" then
                    return( UNKNOWN )
                elif has(term,x) then
                    return( FALSE)
                else
                    return( is(term=0) )
                end_if
             end_if
         end_case
    end_if
end_proc:

